@@ -29,9 +29,9 @@ public class Frame : MarshalByRefObject, IDisposable
 {
 	// Internal state.
 	private Image image;
-	private int width;
-	private int height;
-	private int stride;
+	internal int width;
+	internal int height;
+	internal int stride;
 	private int maskStride;
 	internal PixelFormat pixelFormat;
 	private int[] palette;
@@ -313,16 +313,20 @@ public class Frame : MarshalByRefObject, IDisposable
 						return b | g << 8 | r << 16;
 					}
 					case (PixelFormat.Format32bppArgb):
+					{
 						ptr += 4 * x;
 						return data[ptr++] | data[ptr++] << 8 | data[ptr++] << 16;
+					}
 					case (PixelFormat.Format32bppRgb):
 					{
 						ptr += 4 * x;
 						return data[ptr] | data[ptr+1] << 8 | data[ptr+2] << 16;
 					}
 					case (PixelFormat.Format24bppRgb):
+					{
 						ptr += 3 * x;
 						return data[ptr++] | data[ptr++] << 8 | data[ptr++] << 16;
+					}
 					case (PixelFormat.Format16bppRgb565):
 					{
 						ptr += 2 * x;
@@ -350,21 +354,29 @@ public class Frame : MarshalByRefObject, IDisposable
 						return all | all << 8 | all << 16;
 					}
 					case (PixelFormat.Format8bppIndexed):
+					{
 						return palette[data[ptr]];
+					}
 					case (PixelFormat.Format4bppIndexed):
+					{
 						ptr += x/2;
 						if ((x & 0x01) == 0)
 							return palette[data[ptr]>>4];
 						else
 							return palette[data[ptr] & 0x0F];
+					}
 					case (PixelFormat.Format1bppIndexed):
+					{
 						ptr += x/8;
 						if ((data[ptr] &(1<<7 - (x & 0x07)))== 0)
 							return palette[0];
 						else
 							return palette[1];
+					}
 					default:
-						throw new NotSupportedException();
+					{
+						throw new NotSupportedException(pixelFormat.ToString());
+					}
 				}
 			}
 
@@ -423,6 +435,9 @@ public class Frame : MarshalByRefObject, IDisposable
 	// Get a re-scaled version of this frame.
 	public Frame Scale(int newWidth, int newHeight)
 			{
+				if ((newWidth == width) && (newHeight == height))
+					return CloneFrame (image);
+
 				return BmpResizer.Resize(this, 0, 0, Width, Height, newWidth, newHeight);
 			}
 
@@ -526,7 +541,7 @@ public class Frame : MarshalByRefObject, IDisposable
 			{
 				// Return if there are no changes.
 				if (originx1 == destx1 && originy1 == desty1 && originx2 == destx2 &&
-					originy2 == originy2 && originx3 == destx3 && originy3 == desty3)
+					originy2 == desty2 && originx3 == destx3 && originy3 == desty3)
 					return this.CloneFrame(null);
 				// TODO
 				if (originx1 != originx3 || originy1 != originy2 || desty2 != 0 || destx3 != 0)
@@ -672,7 +687,7 @@ public class Frame : MarshalByRefObject, IDisposable
 
 				int destXOffset = x * bits / 8;
 				int pDestinationRow = y * destStride + destXOffset;
-				int lineLength = width /8 * bits;
+				int lineLength = (width * bits + 7) / 8;
 				
 				if (bits >= 8)
 				{
@@ -850,7 +865,7 @@ public class Frame : MarshalByRefObject, IDisposable
 						{
 							case PixelFormat.Format32bppArgb:
 							{
-								alpha = data[offset + 4 * x] ; 
+								alpha = data[offset + 4 * x + 3] ; 
 							}
 							break;
 							case PixelFormat.Format32bppPArgb:
