@@ -76,10 +76,20 @@ internal unsafe sealed class GNUTLS : ISecureSessionProvider
 	{
 		// Internal state.
 		protected IntPtr cred;
+		private bool isDisposed;
 
 		// Constructor.
 		protected GNUTLSCredentials() {}
 
+		// Finalizer.
+		~GNUTLSCredentials()
+		{
+			if(isDisposed == false)
+			{
+				Dispose();
+			}
+		}
+
 		// Get the credential pointer.
 		public IntPtr Cred
 				{
@@ -96,6 +106,7 @@ internal unsafe sealed class GNUTLS : ISecureSessionProvider
 		public virtual void Dispose()
 				{
 					cred = IntPtr.Zero;
+					isDisposed = true;
 				}
 
 	}; // class GNUTLSCredentials
@@ -103,6 +114,8 @@ internal unsafe sealed class GNUTLS : ISecureSessionProvider
 	// Credentials object that is based on a certificate/private key pair.
 	private sealed class GNUTLSCertificateCredentials : GNUTLSCredentials
 	{
+		private bool isDisposed;
+
 		// Constructor.
 		public GNUTLSCertificateCredentials(byte[] certificate, byte[] key)
 				{
@@ -115,6 +128,15 @@ internal unsafe sealed class GNUTLS : ISecureSessionProvider
 					k.Free();
 				}
 
+		// Finalizer.
+		~GNUTLSCertificateCredentials()
+				{
+					if(isDisposed == false)
+					{
+						Dispose();
+					}
+				}
+
 		// Get the credential type.
 		public override int CredType
 				{
@@ -139,12 +161,23 @@ internal unsafe sealed class GNUTLS : ISecureSessionProvider
 	// Credentials object for anonymous access.
 	private sealed class GNUTLSAnonCredentials : GNUTLSCredentials
 	{
+		private bool isDisposed;
+
 		// Constructor.
 		public GNUTLSAnonCredentials()
 				{
 					gnutls_certificate_allocate_credentials(out cred);
 				}
 
+		// Finalizer.
+		~GNUTLSAnonCredentials()
+				{
+					if(isDisposed == false)
+					{
+						Dispose();
+					}
+				}
+
 		// Get the credential type.
 		public override int CredType
 				{
@@ -162,6 +195,7 @@ internal unsafe sealed class GNUTLS : ISecureSessionProvider
 						gnutls_certificate_free_credentials(cred);
 						base.Dispose();
 					}
+					isDisposed = true;
 				}
 
 	}; // class GNUTLSAnonCredentials
@@ -389,7 +423,8 @@ internal unsafe sealed class GNUTLS : ISecureSessionProvider
 		// Implement the IDisposable interface.
 		public void Dispose()
 				{
-					// Nothing to do in this implemementation.
+					if(stream != null)
+						stream.Close();
 				}
 
 	}; // class GNUTLSSession
@@ -646,10 +681,11 @@ internal unsafe sealed class GNUTLS : ISecureSessionProvider
 			 ref gnutls_datum key, Int type);
 
 	[StructLayout(LayoutKind.Sequential)]
-	private struct gnutls_datum
+	private struct gnutls_datum : IDisposable
 	{
 		public IntPtr data;
 		public UInt size;
+		private bool isDisposed;
 
 		// Constructor.
 		public gnutls_datum(byte[] value)
@@ -659,6 +695,27 @@ internal unsafe sealed class GNUTLS : ISecureSessionProvider
 					size = (UInt)(value.Length);
 				}
 
+		// Finalizer.
+		// structs do not normally have finalizers, however
+		// this one has unmanaged resources that needs to be freed
+		~gnutls_datum()
+				{
+					if(isDisposed == false)
+					{
+						Free();
+					}
+				}
+
+		// IDisposable interface because we have unmanaged resources
+		public void Dispose()
+				{
+					if(isDisposed == false)
+					{
+						Free();
+					}
+					isDisposed = true;
+				}
+
 		// Free the data in this object.
 		public void Free()
 				{
