@@ -31,6 +31,9 @@ internal abstract class DrawingWindow : IToolkitWindow
 	protected internal IntPtr hwnd;
 	protected internal IntPtr hdc;
 	protected internal DrawingWindow parent;
+	
+	private static Win32.Api.WindowsMessages registeredBeginInvokeMessage = 0;
+
 	// The window at the top of the hierarchy
 	protected internal DrawingWindow topOfhierarchy;
 	private Color background;
@@ -59,6 +62,9 @@ internal abstract class DrawingWindow : IToolkitWindow
 	protected DrawingWindow( DrawingToolkit toolkit )
 	{
 		this.toolkit = toolkit;
+		if( registeredBeginInvokeMessage == 0 )
+			registeredBeginInvokeMessage =
+				Win32.Api.RegisterWindowMessageA("DOTNET_BEGIN_INVOKE_MESSAGE");
 	}
 
 	// Move this window underneath the last window with the same parent.
@@ -599,12 +605,18 @@ internal abstract class DrawingWindow : IToolkitWindow
 		return FindDeepestChild(lParam, out x, out y, out actual);
 	}
 
-	internal bool KeyDown( int wParam, int lParam)
+  	internal bool KeyDown( int wParam, int lParam)
 	{
 		//Console.WriteLine("DrawingWindow.KeyDown " + sink +", " + (MapKeyToToolkitKeys( wParam)).ToString() + " " + wParam);
 		return sink.ToolkitKeyDown(MapKeyToToolkitKeys( wParam));
 	}
 
+	internal bool SendBeginInvoke( int wParam )
+	{
+		sink.ToolkitBeginInvoke( (IntPtr)wParam );
+		return true;
+	}
+
 	internal bool Char( int wParam, int lParam)
 	{
 		//Console.WriteLine("DrawingWindow.Char " + sink +", "+ ((char)wParam).ToString());
@@ -673,7 +685,7 @@ internal abstract class DrawingWindow : IToolkitWindow
 			//TODO Pointers are not available yet
 			//Win32.Api.WINDOWPOS *pos = (Win32.Api.WINDOWPOS*)lParam;
 			Win32.Api.WINDOWPOS pos = (Win32.Api.WINDOWPOS)Marshal.PtrToStructure(new IntPtr(lParam), typeof(Win32.Api.WINDOWPOS));
-			if (suspendExternalMoveResizeNotify == true)
+			if (suspendExternalMoveResizeNotify)
 				return;
 			// If moving
 			if (((Win32.Api.SetWindowsPosFlags)pos.flags & Win32.Api.SetWindowsPosFlags.SWP_NOMOVE) == 0)
@@ -851,6 +863,11 @@ internal abstract class DrawingWindow : IToolkitWindow
 			if (toolkit.windows[to] == toWindow)
 				break;
 		}
+		if(to == toolkit.windowCount)
+		{
+			to--;
+		}
+
 		if (from == to + 1 || from == to)
 			return;
 		DrawingWindow[] move = new DrawingWindow[16];
@@ -909,5 +926,17 @@ internal abstract class DrawingWindow : IToolkitWindow
 		return false;
 	}
 
+	void IToolkitWindow.SendBeginInvoke(IntPtr i_gch)
+	{
+		StaticSendBeginInvoke(hwnd,i_gch);
+	}
+
+	public static void StaticSendBeginInvoke(IntPtr hwnd_rcv, IntPtr i_gch)
+	{
+		Win32.Api.PostMessageA(hwnd_rcv,
+			registeredBeginInvokeMessage,
+			(int)i_gch,(int)0);
+	}
+
 }
 }
