@@ -45,9 +45,9 @@ internal sealed class DrawingPen : ToolkitPenBase
 				switch(style)
 				{
 					case DashStyle.Solid:
-					case DashStyle.Custom:
 					default:
 						return LineStyle.LineSolid;
+					case DashStyle.Custom:
 					case DashStyle.Dash:
 					case DashStyle.Dot:
 					case DashStyle.DashDot:
@@ -96,9 +96,12 @@ internal sealed class DrawingPen : ToolkitPenBase
 	// Select this pen into a graphics object.
 	public override void Select(IToolkitGraphics _graphics)
 			{
-				DrawingGraphics graphics = (_graphics as DrawingGraphics);
-				if(graphics != null)
+				if (_graphics == null)
+					return;
+				
+				if (_graphics is DrawingGraphics)
 				{
+					DrawingGraphics graphics = _graphics as DrawingGraphics;
 					Xsharp.Graphics g = graphics.graphics;
 					int width = (int)(properties.Width);
 					LineStyle style = MapLineStyle(properties.DashStyle);
@@ -133,9 +136,27 @@ internal sealed class DrawingPen : ToolkitPenBase
 								g.DashPattern = dashdotdot;
 							}
 							break;
+							
+							case DashStyle.Custom :
+							{
+								float [] src = properties.DashPattern;
+								int iLen = src.Length;
+								byte [] ayCopy = new byte[ iLen ];
+								float fWidth = properties.Width;
+								float tmp;
+								for( int i = 0; i < iLen; i++ ) {
+									tmp = src[i]*fWidth;
+									     if( tmp < 0    ) tmp = 0;
+									else if( tmp > 0xFF ) tmp = 0xFF;
+									ayCopy[i] = (byte) ( tmp );
+									if( ayCopy[i] == 0 ) ayCopy[i] = 1; // must not be zero
+								}
+								g.DashPattern = ayCopy; 
+							}
+							break;
 						}
 					}
-					g.Function = Function.GXcopy;
+					g.Function = GCFunction.GXcopy;
 					g.SubwindowMode = SubwindowMode.ClipByChildren;
 					g.LineWidth = width;
 					g.LineStyle = style;
@@ -146,6 +167,11 @@ internal sealed class DrawingPen : ToolkitPenBase
 					g.SetFillSolid();
 					graphics.Pen = this;
 				}
+				else if (_graphics is DrawingGraphicsImage)
+				{
+					DrawingGraphicsImage graphics = _graphics as DrawingGraphicsImage;
+					graphics.Pen = this;
+				}
 			}
 
 	// Select a brush-based pen into a graphics object.
