@@ -183,16 +183,18 @@ internal sealed class DrawingPopupWindow : PopupWindow, IToolkitWindow
 				}
 			}
 
+	[TODO]
 	// Move this window to above one of its siblings.
 	void IToolkitWindow.MoveToAbove(IToolkitWindow sibling)
 			{
-				// TODO
+				return;
 			}
 
+	[TODO]
 	// Move this window to below one of its siblings.
 	void IToolkitWindow.MoveToBelow(IToolkitWindow sibling)
 			{
-				// TODO
+				return;
 			}
 
 	// Get the HWND for this window.  IntPtr.Zero if not supported.
@@ -389,6 +391,11 @@ internal sealed class DrawingPopupWindow : PopupWindow, IToolkitWindow
 			}
 
 
+	void IToolkitWindow.SendBeginInvoke(IntPtr i_gch)
+		{
+			base.SendBeginInvoke(i_gch);
+		}
+
 }; // class DrawingPopupWindow
 
 }; // namespace System.Drawing.Toolkit
