@@ -24,6 +24,7 @@ namespace System.Drawing.Toolkit
 using System;
 using System.Globalization;
 using System.Collections;
+using System.Diagnostics;
 using System.Drawing;
 using System.Drawing.Drawing2D;
 using System.Drawing.Toolkit;
@@ -188,25 +189,28 @@ public sealed class DrawingToolkit : IToolkit
 				app.Display.DefaultScreenOfDisplay.Wakeup();
 			}
 
+	[TODO]
 	// Get lighter or darker versions of a color.
 	private static int Light(int color)
 			{
-				// TODO
 				return -1;
 			}
+
+	[TODO]
 	private static int LightLight(int color)
 			{
-				// TODO
 				return -1;
 			}
+
+	[TODO]
 	private static int Dark(int color)
 			{
-				// TODO
 				return -1;
 			}
+
+	[TODO]
 	private static int DarkDark(int color)
 			{
-				// TODO
 				return -1;
 			}
 
@@ -732,10 +736,10 @@ public sealed class DrawingToolkit : IToolkit
 					(app.Display.DefaultScreenOfDisplay, image, frame);
 			}
 
+	[TODO]
 	// Get the clipboard handler for this toolkit, or null if no clipboard.
 	public IToolkitClipboard GetClipboard()
 			{
-				// TODO
 				return null;
 			}
 
