@@ -199,13 +199,15 @@ internal sealed class DrawingTopLevelWindow
 	// Move this window to above one of its siblings.
 	void IToolkitWindow.MoveToAbove(IToolkitWindow sibling)
 			{
-				// Not used for top-level windows.
+				// Move this window below the sibling widget.
+				MoveToAbove(sibling as Widget);
 			}
 
 	// Move this window to below one of its siblings.
 	void IToolkitWindow.MoveToBelow(IToolkitWindow sibling)
 			{
-				// Not used for top-level windows.
+				// Move this window below the sibling widget.
+				MoveToBelow(sibling as Widget);
 			}
 
 	// Get the HWND for this window.  IntPtr.Zero if not supported.
@@ -225,7 +227,7 @@ internal sealed class DrawingTopLevelWindow
 			{
 				DrawingToolkit.ValidateWindowPosition(ref x, ref y);
 				DrawingToolkit.ValidateWindowSize(ref width, ref height);
-				Repaint(x, y, width, height);
+				Repaint(x, y, width + 1, height + 1);
 			}
 
 	// Force an update of all invalidated regions.
@@ -380,7 +382,7 @@ internal sealed class DrawingTopLevelWindow
 				{
 					otherHints |= OtherHints.Dialog;
 				}
-				if((flags & ToolkitWindowFlags.ShowInTaskbar) != 0)
+				if((flags & ToolkitWindowFlags.ShowInTaskbar) == 0)
 				{
 					otherHints |= OtherHints.HideFromTaskBar;
 				}
@@ -408,6 +410,11 @@ internal sealed class DrawingTopLevelWindow
 				OtherHints = otherHints;
 			}
 
+	void IToolkitTopLevelWindow.SetOpacity(double opacity)
+			{
+				Opacity = opacity;
+			}
+
 	protected override void OnBeginInvokeMessage(IntPtr i_gch)
 			{
 				if( sink != null )
@@ -658,6 +665,59 @@ internal sealed class DrawingTopLevelWindow
 			{
 				base.SendBeginInvoke(i_gch);
 			}
+			
+	protected override bool OnButtonWheel(int x, int y, ButtonName button,
+									  	   ModifierMask modifiers, int iDelta)
+			{
+				if(sink != null)
+				{
+					sink.ToolkitMouseWheel
+						(MapButton(button),
+						MapKey(KeyName.XK_VoidSymbol, modifiers),
+						1, x, y, iDelta);
+				}
+
+				return true;
+			}
+			
+	// Map an Xsharp key description into a "ToolkitKeys" value.
+	internal static ToolkitKeys MapKey(KeyName key, ModifierMask modifiers)
+			{
+				ToolkitKeys toolkitKey =  ToolkitKeys.None;
+				if((modifiers & ModifierMask.ControlMask) != 0)
+				{
+					toolkitKey |= ToolkitKeys.Control;
+				}
+				if((modifiers & ModifierMask.ShiftMask) != 0)
+				{
+					toolkitKey |= ToolkitKeys.Shift;
+				}
+				if((modifiers & ModifierMask.Mod1Mask) != 0)
+				{
+					toolkitKey |= ToolkitKeys.Alt;
+				}
+				return toolkitKey;
+			}
+
+	// Map an Xsharp button name into a "ToolkitMouseButtons" value.
+	internal static ToolkitMouseButtons MapButton(ButtonName button)
+			{
+				switch(button)
+				{
+					case ButtonName.Button1:
+						return ToolkitMouseButtons.Left;
+					case ButtonName.Button2:
+						return ToolkitMouseButtons.Middle;
+					case ButtonName.Button3:
+						return ToolkitMouseButtons.Right;
+					case ButtonName.Button4:
+						return ToolkitMouseButtons.XButton1;
+					case ButtonName.Button5:
+						return ToolkitMouseButtons.XButton2;
+					default:
+						return ToolkitMouseButtons.None;
+				}
+			}	
 
 }; // class DrawingTopLevelWindow
 
