@@ -125,6 +125,12 @@ internal sealed class DrawingWindow : InputOutputWidget, IToolkitWindow
 	void IToolkitWindow.Destroy()
 			{
 				Destroy();
+				/*
+				 * if the window is destroyed, set toolkit and sink to zero.
+				 * otherwise, we couold get events like LostFocus...
+				*/
+				toolkit = null;
+				sink    = null;
 			}
 
 	// Move or resize this window.
@@ -658,8 +664,10 @@ internal sealed class DrawingWindow : InputOutputWidget, IToolkitWindow
 				{
 					// Create a cursor based on the supplied image.
 					if(frame != null &&
-					   frame.PixelFormat ==
-					   		DotGNU.Images.PixelFormat.Format1bppIndexed &&
+									 /* irgnore pixel format !
+									 frame.PixelFormat == DotGNU.Images.PixelFormat.Format1bppIndexed 
+									 &&
+									 */
 					   frame.Mask != null)
 					{
 						if(cursorType != ToolkitCursorType.UserDefined)
