@@ -19,14 +19,19 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+using System.Collections;
+
 namespace System.Drawing.Drawing2D
 {
 
 public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 {
 	// Internal state.
-	private PathObject first;
-	private PathObject last;
+	private ArrayList  pathFigures  = new ArrayList(50);
+	private ArrayList  stringObjs   = new ArrayList(50);
+	private PointF     []pathPoints = new PointF[0];
+	private PathFigure actualFigure = null; 
+
 	private bool needPenBrush;
 	private FillMode fillMode;
 
@@ -129,8 +134,8 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 				{
 					PathData data = new PathData();
 					// TODO
-					//data.Points = pts;
-					//data.Types = types;
+					data.Points = this.pathPoints;
+					data.Types = this.PathTypes;
 					return data;
 				}
 			}
@@ -139,9 +144,7 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 			{
 				get
 				{
-					// TODO
-					//return pts;
-					return null;
+					return this.pathPoints;
 				}
 			}
 	[TODO]
@@ -151,7 +154,7 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 				{
 					// TODO
 					//return types;
-					return null;
+					return new byte [this.pathPoints.Length];
 				}
 			}
 
@@ -161,23 +164,28 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 			{
 				get
 				{
-					// TODO
-					return 0;
+					return this.pathPoints.Length;
 				}
 			}
+			
+	private void AddPathPoints( PointF [] pts ) 
+		{
+			if( null == pts || pts.Length == 0 ) return;
+			PointF [] pNew = new PointF[this.pathPoints.Length+pts.Length];
+			Array.Copy( this.pathPoints, 0, pNew, 0, this.pathPoints.Length );
+			Array.Copy( pts, 0, pNew, this.pathPoints.Length, pts.Length );
+			this.pathPoints = pNew;
+		}
 
 	// Add an object to this path.
 	private void AddPathObject(PathObject obj)
 			{
-				if(last != null)
-				{
-					last.next = obj;
+				if( null == actualFigure ) {
+					actualFigure = new PathFigure();
+					pathFigures.Add( actualFigure );
 				}
-				else
-				{
-					first = obj;
-				}
-				last = obj;
+				actualFigure.AddPathObject( obj );
+				this.AddPathPoints( obj.GetPathPoints() );
 			}
 
 	// Add an arc to the current figure.
@@ -284,7 +292,9 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 						(String.Format
 							(S._("Arg_NeedsAtLeastNPoints"), 4));
 				}
+				this.actualFigure = null; // Close Figure before adding a closed curve
 				AddPathObject(new ClosedCurvePathObject(points, tension));
+				this.actualFigure = null; // Close Figure after adding a closed curve
 			}
 
 	// Add a curve to the current path.
@@ -364,8 +374,10 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 			}
 	public void AddEllipse(float x, float y, float width, float height)
 			{
+				this.actualFigure = null;	// Ellipse closes figure
 				AddPathObject(new ArcPathObject
-					(x, y, width, height, 0.0f, 360f));
+					(x, y, width, height, -5f, 365f));
+				this.actualFigure = null;	// Ellipse closes figure
 			}
 
 	// Add a line to the current figure.
@@ -410,19 +422,27 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 			}
 
 	// Append another path to this one.  "connect" is intended for figures,
-	// which we don't do anything special with here.
 	public void AddPath(GraphicsPath addingPath, bool connect)
 			{
 				if(addingPath == null)
 				{
 					throw new ArgumentNullException("addingPath");
 				}
-				PathObject obj = addingPath.first;
-				while(obj != null)
-				{
-					AddPathObject(obj.Clone());
-					obj = obj.next;
+				
+				if( connect ) {
+					foreach( PathFigure fig in addingPath.pathFigures ) {
+						foreach( PathObject obj in fig.pathObjs ) {
+							this.AddPathObject( obj.Clone() );
+						}
+					}
 				}
+				else {
+					foreach( PathFigure fig in addingPath.pathFigures ) {
+						this.pathFigures.Add( fig.Clone() );
+					}
+					actualFigure = null;
+				}
+				
 				if(addingPath.needPenBrush)
 				{
 					needPenBrush = true;
@@ -445,8 +465,10 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 	public void AddPie(float x, float y, float width, float height,
 					   float startAngle, float sweepAngle)
 			{
+				this.actualFigure = null; // Adding Pie closes a figure
 				AddPathObject(new PiePathObject(x, y, width, height,
 												startAngle, sweepAngle));
+				this.actualFigure = null;// Adding Pie closes a figure
 			}
 
 	// Add a polygon to this path.
@@ -466,7 +488,9 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 						(String.Format
 							(S._("Arg_NeedsAtLeastNPoints"), 2));
 				}
+				this.actualFigure = null; // Adding Plygon closes a figure
 				AddPathObject(new PolygonPathObject(points));
+				this.actualFigure = null; // Adding Plygon closes a figure
 			}
 
 	// Add a rectangle to this path.
@@ -476,8 +500,10 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 			}
 	public void AddRectangle(RectangleF rect)
 			{
+				this.actualFigure = null; // Adding Rectangle closes a figure
 				AddPathObject(new RectanglePathObject
 					(rect.X, rect.Y, rect.Width, rect.Height));
+				this.actualFigure = null; // Adding Rectangle closes a figure
 			}
 
 	// Add a list of rectangles to this path.
@@ -510,7 +536,8 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 						  int style, float emSize,
 						  PointF origin, StringFormat format)
 			{
-				// TODO
+				// String Paths are added drawn
+				this.stringObjs.Add( new StringPathObject( s, family, style, emSize, origin, format ) );
 			}
 	public void AddString(String s, FontFamily family,
 						  int style, float emSize,
@@ -524,7 +551,9 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 						  int style, float emSize,
 						  RectangleF layoutRect, StringFormat format)
 			{
-				// TODO
+				this.actualFigure = null; // Close Figure before adding a Text
+				AddPathObject( new StringPathObject( s, family, style, emSize, layoutRect, format ) );
+				this.actualFigure = null; // Close Figure after adding a Text
 			}
 
 	// Clean all markers from this path.
@@ -538,11 +567,9 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 			{
 				GraphicsPath path = new GraphicsPath(fillMode);
 				path.needPenBrush = needPenBrush;
-				PathObject obj = first;
-				while(obj != null)
-				{
-					path.AddPathObject(obj.Clone());
-					obj = obj.next;
+				
+				foreach( PathFigure figure in pathFigures ) {
+					path.pathFigures.Add( figure.Clone() );
 				}
 				return path;
 			}
@@ -550,13 +577,13 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 	// Close all figures within the path.
 	public void CloseAllFigures()
 			{
-				// We don't do anything special with figures here.
+				actualFigure = null;
 			}
 
 	// Close the current figure and start a new one.
 	public void CloseFigure()
 			{
-				// We don't do anything special with figures here.
+				actualFigure = null;
 			}
 
 	// Dispose of this object.
@@ -605,6 +632,7 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 	public PointF GetLastPoint()
 			{
 				PointF[] points = PathPoints;
+				if( null == PathPoints ) return new PointF(0,0);
 				return points[points.Length - 1];
 			}
 
@@ -685,8 +713,9 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 	// Reset this path.
 	public void Reset()
 			{
-				first = null;
-				last = null;
+				pathPoints = new PointF[0]; // reset path points
+				actualFigure = null;
+				pathFigures.Clear();
 				needPenBrush = false;
 				fillMode = FillMode.Alternate;
 			}
@@ -694,20 +723,10 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 	// Reverse the order of the path.
 	public void Reverse()
 			{
-				PathObject obj = first;
-				PathObject next;
-				first = null;
-				last = null;
-				while(obj != null)
-				{
-					next = obj.next;
-					obj.next = first;
-					first = obj;
-					if(last == null)
-					{
-						last = obj;
-					}
+				foreach( PathFigure fig in pathFigures ) {
+					fig.Reverse();
 				}
+				pathFigures.Reverse();
 			}
 
 	// Set a marker.
@@ -720,13 +739,19 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 	public void StartFigure()
 			{
 				// We don't do anything special with figures here.
+				actualFigure = null;
 			}
 
 	// Apply a transformation to this path.
 	[TODO]
 	public void Transform(Matrix matrix)
 			{
-				// TODO
+				foreach( PathFigure fig in pathFigures ) {
+					fig.Transform(matrix);
+				}
+				foreach( StringPathObject obj in stringObjs ) {
+					obj.Transform(matrix);
+				}
 			}
 
 	// Apply a warp transformation to this path.
@@ -767,12 +792,12 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 
 	// Draw this graphics path.
 	internal void Draw(Graphics graphics, Pen pen)
-			{
-				PathObject obj = first;
-				while(obj != null)
-				{
-					obj.Draw(graphics, pen);
-					obj = obj.next;
+			{	
+				foreach( PathFigure fig in pathFigures ) {
+					fig.Draw( graphics, pen );
+				}
+				foreach( StringPathObject obj in stringObjs ) {
+					obj.Draw( graphics, pen );
 				}
 			}
 
@@ -788,23 +813,139 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 				{
 					pen = null;
 				}
-				PathObject obj = first;
-				while(obj != null)
-				{
-					obj.Fill(graphics, brush, pen, fillMode);
-					obj = obj.next;
+				
+				// XorBrush xorbrush = new XorBrush( brush );
+				
+				/* TODO:
+					Fill graphics Path xor, does not do yet :(
+				*/
+				
+				foreach( PathFigure fig in pathFigures ) {
+					fig.Fill( graphics, brush, pen, fillMode );
 				}
+				foreach( StringPathObject obj in stringObjs ) {
+					obj.Fill( graphics, brush, pen, fillMode );
+				}
+				
+				/*
+				XorBrush xorbrush = new XorBrush( brush );
+				foreach( PathFigure fig in pathFigures ) {
+					fig.Fill( graphics, xorbrush, pen, fillMode );
+				}
+				foreach( StringPathObject obj in stringObjs ) {
+					obj.Fill( graphics, xorbrush, pen, fillMode );
+				}
+				*/
+				
 				if(needPenBrush)
 				{
 					pen.Dispose();
 				}
 			}
+			
+	private class PathFigure : ICloneable
+	{
+		public ArrayList pathObjs;
+		
+		public PathFigure() 
+		{
+			pathObjs = new ArrayList();
+		}
+		
+		public object Clone() {
+			PathFigure cpy = new PathFigure();
+			foreach( PathObject o in pathObjs ) {
+				cpy.AddPathObject( o.Clone() );
+			}
+			return cpy;
+		}
+		
+		public PointF[] GetPathPoints() {
+			PointF[][] fpoints = new PointF[pathObjs.Count][];
+			int size=0;
+			int i= 0;
+			foreach( PathObject obj in pathObjs ) 
+			{
+				fpoints[i] = obj.GetPathPoints();
+				if( null != fpoints[i] ) size += fpoints[i].Length;
+				i++;
+			}
+				
+			PointF[] poly= new PointF[size];
+			int z=0;
+			for(i=0;i<fpoints.Length;i++) {
+				if( null != fpoints[i] ) {
+					for(int j=0;j<fpoints[i].Length;j++) {
+						poly[z++]=fpoints[i][j];
+					}
+				}
+			}
+				
+			return poly;
+		}
+		
+		public void AddPathObject(PathObject obj)
+		{
+			pathObjs.Add( obj );
+		}
+		
+		public void Reverse() {
+			pathObjs.Reverse();
+		} 
+		
+		public void Transform(Matrix matrix)
+		{
+			foreach( PathObject obj in pathObjs ) {
+				obj.Transform( matrix );
+			}
+		}
+		
+		// Draw the graphics path
+		public void Draw(Graphics graphics, Pen pen)
+		{
+			/*
+			foreach( PathObject o in pathObjs ) {
+				o.Draw( graphics, pen );
+			}
+			*/
+			
+			PointF [] poly = GetPathPoints();
+				
+			if( poly.Length > 1 ) {
+				
+				graphics.DrawLines(pen, poly);
+				
+			}
+			foreach( PathObject o in pathObjs ) {
+				if( !o.HasPathPoints ) o.Draw( graphics, pen );
+			}
+		}
+
+		// Fill this graphics path.
+		public void Fill(Graphics graphics, Brush brush, Pen pen, FillMode fillMode)
+		{
+			/*
+			foreach( PathObject o in pathObjs ) {
+				o.Fill( graphics, brush, pen, fillMode );
+			}
+			*/
+			PointF [] poly = GetPathPoints();
+				
+			if( poly.Length > 2 ) {
+				
+				graphics.FillPolygon(brush, poly, fillMode );
+				
+			}
+			foreach( PathObject o in pathObjs ) {
+				if( !o.HasPathPoints ) o.Fill(graphics, brush, pen, fillMode);
+			}
+		}
+	}
 
 	// Base class for path objects.
 	private abstract class PathObject
 	{
-		public PathObject next;
-
+		protected Matrix mMatrix;
 		// Constructor.
 		protected PathObject() {}
 
@@ -817,7 +958,143 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 
 		// Clone this path object.
 		public abstract PathObject Clone();
+		
+		public void Transform(Matrix matrix)
+		{
+			if( null == mMatrix ) {
+				mMatrix = new Matrix();
+			}
+			mMatrix.Multiply( matrix );
+		}
+		
+		public PointF[] GetPathPoints() {
+			PointF[] data = DoGetPathPoints();
+			if( null == data ) return null;
+			data = (PointF[])data.Clone();
+			if( null != mMatrix ) {
+				mMatrix.TransformPoints( data );
+			}
+			return data;
+		}
+		protected abstract PointF [] DoGetPathPoints();
+		public abstract bool HasPathPoints { get; }
 
+		protected static PointF ComputePoint(double u,
+														 double x1, double y1,
+														 double x2, double y2,
+														 double x3, double y3,
+														 double x4, double y4)
+		{
+			double px=0;
+			double py=0;
+			double blend;
+			double u2=(1.0-u);
+
+				// Point 0
+			blend = u2*u2*u2;
+			px += blend * x1;
+			py += blend * y1;
+
+				// Point 1
+			blend = 3 * u * u2 * u2;
+			px += blend * x2;
+			py += blend * y2;
+
+				// Point 2
+			blend = 3 * u * u * u2;
+			px += blend * x3;
+			py += blend * y3;
+
+				// Point 3
+			blend = u * u * u;
+			px += blend * x4;
+			py += blend * y4;
+
+			return new PointF((float)Math.Round(px),(float)Math.Round(py));
+		}
+
+		protected int ComputeSteps(double x1, double y1, double x2, double y2, double x3,
+											double y3, double x4, double y4)
+		{
+			double length=0L;
+				
+			length+=Math.Sqrt((x1-x2)*(x1-x2) + (y1-y2) * (y1-y2));
+			length+=Math.Sqrt((x2-x3)*(x2-x3) + (y2-y3) * (y2-y3));
+			length+=Math.Sqrt((x3-x4)*(x3-x4) + (y3-y4) * (y3-y4));
+				
+			return (int)Math.Ceiling(length);	
+		}
+
+		protected PointF[] ComputeBezier(double x1, double y1, double x2, double y2,		
+													double x3, double y3, double x4, double y4)
+		{
+			int steps=ComputeSteps(x1,y1,x2,y2,x3,y3,x4,y4);
+			PointF [] points=new PointF[steps];
+			for(int i=0;i<steps;i++)
+			{
+				double coeff=((double)i+1)/steps;
+				points[i]=ComputePoint(coeff ,x1,y1,x2,y2,x3,y3,x4,y4);
+			}                	
+			return points;
+		}
+
+		protected PointF[] ComputeSplineSegment(PointF p0,PointF p1,PointF T1, PointF T2 )
+		{                           
+			int steps=ComputeSteps(p0.X,p0.Y,T1.X,T1.Y,T2.X,T2.Y,p1.X,p1.Y);			
+			PointF[] points = new PointF[steps+1];
+			for(int i=0;i<=steps;i++)
+			{
+				double s=((double)i)/steps;
+				double s2=s*s;
+				double s3=s2*s;
+				double h1 =  2*s3 - 3*s2 + 1;
+				double h2 = -2*s3 + 3*s2;
+				double h3 =    s3 - 2*s2 + s;
+				double h4 =    s3 -   s2;   
+				points[i].X=(float)(h1*p0.X+h2*p1.X+h3*T1.X+h4*T2.X);
+				points[i].Y=(float)(h1*p0.Y+h2*p1.Y+h3*T1.Y+h4*T2.Y);
+			}                	
+			return points;
+		}
+               
+		protected PointF[] ComputeTangent(PointF[] points, float tension, bool closed, int numberOfSegments)
+		{
+			PointF p0,p1;
+			if (numberOfSegments<3) return null;
+			PointF[] tan=new PointF[numberOfSegments];	
+			for(int i=0;i<numberOfSegments;i++)
+			{                       
+				if(i==0) 
+				{
+					if(closed)
+						p0 = points[numberOfSegments-1];
+					else
+						p0 = points[0];
+				} else p0 = points[i-1];
+				if(i==numberOfSegments-1)
+				{
+					if(closed)
+						p1 = points[0];
+					else
+						p1 = points[i];
+				} else p1 = points[i+1];
+
+				tan[i].X = (int)(tension*(p1.X - p0.X));
+				tan[i].Y = (int)(tension*(p1.Y - p0.Y));			
+			}			
+			return tan;
+		}
+		
+		protected double DegToRad( double val ) {
+			return Math.PI*val/180.0;
+		}
+		protected double DegCosinus( double val ) {
+			return Math.Cos( DegToRad( val ) );
+		}
+		protected double DegSinus( double val ) {
+			return Math.Sin( DegToRad( val ) );
+		}
+	
 	}; // class PathObject
 
 	// Arc path objects.
@@ -826,6 +1103,8 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 		// Internal state.
 		private float x, y, width, height;
 		private float startAngle, sweepAngle;
+		PointF [] pathPoints;
+		PointF startPoint, endPoint; 
 
 		// Constructor.
 		public ArcPathObject(float x, float y, float width, float height,
@@ -837,21 +1116,56 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 					this.height = height;
 					this.startAngle = startAngle;
 					this.sweepAngle = sweepAngle;
+					this.pathPoints = CalculatePathPoints();
 				}
+				
+		public override bool HasPathPoints { 
+			get {
+				return true;
+			} 
+		}
+		
+		PointF [] CalculatePathPoints() {
+			int    iCount = 360;
+			PointF [] points = new PointF[iCount];
+			
+			double dDelta = this.sweepAngle;
+			dDelta /= iCount;
+			double dStart = this.startAngle;
+			double dEnd   = dStart +  this.sweepAngle;
+			double px,py;
+			double rx = this.width/2;
+			double ry = this.height/2;
+			for( int i = 0; i < iCount-1; i++ ) {
+				px = rx*( 1 + DegCosinus( dStart ) ) + this.x+1;
+				py = ry*( 1 + DegSinus  ( dStart ) ) + this.y+1;
+				points[i] = new PointF( (float)px,(float)py );
+				if( i == 0 ) startPoint = points[i];
+				dStart += dDelta;
+			}
+			px = rx*( 1 + DegCosinus( dEnd ) ) + this.x+1;
+			py = ry*( 1 + DegSinus  ( dEnd ) ) + this.y+1;
+			points[iCount-1] = new PointF( (float)px,(float)py );
+			endPoint = points[iCount-1];
+			return points;
+		}
 
+		protected override PointF [] DoGetPathPoints() {
+			return this.pathPoints;
+		}
+				
 		// Draw this path object.
 		public override void Draw(Graphics graphics, Pen pen)
 				{
-					graphics.DrawArc(pen, x, y, width, height,
-									 startAngle, sweepAngle);
+					graphics.DrawArc(pen, x, y, width, height, startAngle, sweepAngle);
+					//graphics.DrawLine(pen, this.startPoint, this.endPoint);
 				}
 
 		// Fill this path object.
 		public override void Fill(Graphics graphics, Brush brush,
 								  Pen penBrush, FillMode fillMode)
 				{
-					graphics.DrawArc(penBrush, x, y, width, height,
-									 startAngle, sweepAngle);
+					graphics.DrawArc(penBrush, x, y, width, height, startAngle, sweepAngle);
 				}
 
 		// Clone this path object.
@@ -868,6 +1182,7 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 	{
 		// Internal state.
 		private float x1, y1, x2, y2, x3, y3, x4, y4;
+		PointF [] points;
 
 		// Constructor.
 		public BezierPathObject(float x1, float y1, float x2, float y2,
@@ -881,8 +1196,19 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 					this.y3 = y3;
 					this.x4 = x4;
 					this.y4 = y4;
+					points = ComputeBezier( x1, y1, x2, y2, x3, y3, x4, y4 );
 				}
 
+		public override bool HasPathPoints { 
+			get {
+				return true;
+			} 
+		}
+		
+		protected override PointF [] DoGetPathPoints() {
+			 return points;
+		}
+		
 		// Draw this path object.
 		public override void Draw(Graphics graphics, Pen pen)
 				{
@@ -893,8 +1219,7 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 		public override void Fill(Graphics graphics, Brush brush,
 								  Pen penBrush, FillMode fillMode)
 				{
-					graphics.DrawBezier
-						(penBrush, x1, y1, x2, y2, x3, y3, x4, y4);
+					graphics.FillBezier(brush, x1, y1, x2, y2, x3, y3, x4, y4, fillMode );
 				}
 
 		// Clone this path object.
@@ -919,19 +1244,45 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 					this.points = (PointF[])(points.Clone());
 					this.tension = tension;
 				}
+				
+				
+		public override bool HasPathPoints { 
+			get {
+				return false;
+			} 
+		}
+		
+		protected override PointF [] DoGetPathPoints() {
+			return null;
+		}
 
 		// Draw this path object.
 		public override void Draw(Graphics graphics, Pen pen)
 				{
-					graphics.DrawClosedCurve(pen, points, tension,
-											 FillMode.Alternate);
+					if( null != mMatrix ) {
+						Matrix save = graphics.Transform;
+						graphics.Transform = mMatrix;
+						graphics.DrawClosedCurve(pen, points, tension, FillMode.Alternate);
+						graphics.Transform = save;
+					}
+					else {
+						graphics.DrawClosedCurve(pen, points, tension, FillMode.Alternate);
+					}
 				}
 
 		// Fill this path object.
 		public override void Fill(Graphics graphics, Brush brush,
 								  Pen penBrush, FillMode fillMode)
 				{
-					graphics.FillClosedCurve(brush, points, fillMode, tension);
+					if( null != mMatrix ) {
+						Matrix save = graphics.Transform;
+						graphics.Transform = mMatrix;
+						graphics.FillClosedCurve(brush, points, fillMode, tension);
+						graphics.Transform = save;
+					}
+					else {
+						graphics.FillClosedCurve(brush, points, fillMode, tension);
+					}
 				}
 
 		// Clone this path object.
@@ -949,6 +1300,7 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 		private PointF[] points;
 		private int offset, numberOfSegments;
 		private float tension;
+		private PointF [] pathPoints;
 
 		// Constructor.
 		public CurvePathObject(PointF[] points, int offset,
@@ -958,21 +1310,52 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 					this.offset = offset;
 					this.numberOfSegments = numberOfSegments;
 					this.tension = tension;
+					this.pathPoints = ComputePathPoints();
 				}
+				
+		private PointF [] ComputePathPoints() {
+			PointF [] tangent=ComputeTangent(points,tension,false,numberOfSegments+1);
+			if (tangent == null)
+			{
+				return points;
+			}
+			PointF[][] fpoints = new PointF[numberOfSegments][];
+			int size=0;
+			for(int i=0;i<numberOfSegments;i++)
+			{
+				fpoints[i]=ComputeSplineSegment(points[i],points[i+1],tangent[i],tangent[i+1]);
+				size+=fpoints[i].Length;				
+			}
 
+			PointF[] poly= new PointF[size];
+			int z=0;
+			for(int i=0;i<numberOfSegments;i++) 			
+				for(int j=0;j<fpoints[i].Length;j++)
+					poly[z++]=fpoints[i][j];
+			return poly;
+		}
+		
+		public override bool HasPathPoints { 
+			get {
+				return true;
+			} 
+		}
+
+		protected override PointF [] DoGetPathPoints() {
+			return this.pathPoints;
+		}
+		
 		// Draw this path object.
 		public override void Draw(Graphics graphics, Pen pen)
 				{
-					graphics.DrawCurve(pen, points, offset,
-									   numberOfSegments, tension);
+					graphics.DrawCurve(pen, points, offset, numberOfSegments, tension);
 				}
 
 		// Fill this path object.
 		public override void Fill(Graphics graphics, Brush brush,
 								  Pen penBrush, FillMode fillMode)
 				{
-					graphics.DrawCurve(penBrush, points, offset,
-									   numberOfSegments, tension);
+					graphics.DrawCurve(penBrush, points, offset, numberOfSegments, tension);
 				}
 
 		// Clone this path object.
@@ -989,6 +1372,7 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 	{
 		// Internal state.
 		private float x1, y1, x2, y2;
+		PointF [] pathPoints;
 
 		// Constructor.
 		public LinePathObject(float x1, float y1, float x2, float y2)
@@ -997,7 +1381,20 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 					this.y1 = y1;
 					this.x2 = x2;
 					this.y2 = y2;
+					pathPoints = new PointF[2];
+					pathPoints[0] = new PointF( x1,y1 );
+					pathPoints[1] = new PointF( x2,y2 );
 				}
+				
+		public override bool HasPathPoints { 
+			get {
+				return true;
+			} 
+		}
+
+		protected override PointF [] DoGetPathPoints() {
+			return pathPoints;
+		}
 
 		// Draw this path object.
 		public override void Draw(Graphics graphics, Pen pen)
@@ -1032,6 +1429,16 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 					this.points = (PointF[])(points.Clone());
 				}
 
+		public override bool HasPathPoints { 
+			get {
+				return true;
+			} 
+		}
+
+		protected override PointF [] DoGetPathPoints() {
+			return points;
+		}
+		
 		// Draw this path object.
 		public override void Draw(Graphics graphics, Pen pen)
 				{
@@ -1059,6 +1466,7 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 		// Internal state.
 		private float x, y, width, height;
 		private float startAngle, sweepAngle;
+		PointF [] pathPoints;
 
 		// Constructor.
 		public PiePathObject(float x, float y, float width, float height,
@@ -1070,11 +1478,49 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 					this.height = height;
 					this.startAngle = startAngle;
 					this.sweepAngle = sweepAngle;
+					this.pathPoints = CalculatePathPoints();
 				}
+				
+		public override bool HasPathPoints { 
+			get {
+				return true;
+			} 
+		}
 
+		protected override PointF [] DoGetPathPoints() {
+			return this.pathPoints;
+		}
+
+		PointF [] CalculatePathPoints() {
+			int    iCount = 360;
+			PointF [] points = new PointF[iCount+2];
+			
+			double dDelta = this.sweepAngle;
+			dDelta /= iCount;
+			double dStart = this.startAngle;
+			double dEnd   = dStart+this.sweepAngle;
+			double xP,yP;
+			double rx = this.width/2;
+			double ry = this.height/2;
+			points[0] = new PointF( (float)(this.x+rx),(float)(this.y+ry) );
+			for( int i = 1; i < iCount; i++ ) {
+				xP = rx*( 1 + DegCosinus( dStart ) ) + this.x+1;
+				yP = ry*( 1 + DegSinus  ( dStart ) ) + this.y+1;
+				points[i] = new PointF( (float)xP,(float)yP );
+				dStart += dDelta;
+			}
+			xP = rx*( 1 + DegCosinus( dEnd ) ) + this.x+1;
+			yP = ry*( 1 + DegSinus  ( dEnd ) ) + this.y+1;
+			points[iCount]   = new PointF( (float)xP,(float)yP );
+			points[iCount+1] = points[0];
+			return points;
+		}
+
+				
 		// Draw this path object.
 		public override void Draw(Graphics graphics, Pen pen)
 				{
+					// TODO brubbel: calculate points and draw lines
 					graphics.DrawPie(pen, x, y, width, height,
 									 startAngle, sweepAngle);
 				}
@@ -1083,6 +1529,7 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 		public override void Fill(Graphics graphics, Brush brush,
 								  Pen penBrush, FillMode fillMode)
 				{
+					// TODO brubbel: calculate points and fill polygon
 					graphics.FillPie(brush, x, y, width, height,
 									 startAngle, sweepAngle);
 				}
@@ -1101,13 +1548,28 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 	{
 		// Internal state.
 		private PointF[] points;
-
+		private PointF[] pathPoints;
+		
 		// Constructor.
 		public PolygonPathObject(PointF[] points)
 				{
 					this.points = (PointF[])(points.Clone());
+					int iCount = this.points.Length;
+					this.pathPoints = new PointF[iCount+1];
+					this.points.CopyTo( this.pathPoints, 0 );
+					this.pathPoints[iCount] = this.pathPoints[0];
 				}
 
+		public override bool HasPathPoints { 
+			get {
+				return true;
+			} 
+		}
+
+		protected override PointF [] DoGetPathPoints() {
+			return this.pathPoints;
+		}
+				
 		// Draw this path object.
 		public override void Draw(Graphics graphics, Pen pen)
 				{
@@ -1134,6 +1596,7 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 	{
 		// Internal state.
 		private float x, y, width, height;
+		private PointF [] pathPoints;
 
 		// Constructor.
 		public RectanglePathObject(float x, float y, float width, float height)
@@ -1142,7 +1605,24 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 					this.y = y;
 					this.width = width;
 					this.height = height;
+					pathPoints = new PointF[5];
+					pathPoints[0] = new PointF( x, y );
+					pathPoints[1] = new PointF( x+width, y );
+					pathPoints[2] = new PointF( x+width, y+height );
+					pathPoints[3] = new PointF( x      , y+height );
+					pathPoints[4] = new PointF( x      , y );
+					
 				}
+		
+		public override bool HasPathPoints { 
+			get {
+				return true;
+			} 
+		}
+
+		protected override PointF [] DoGetPathPoints() {
+			return pathPoints;
+		}
 
 		// Draw this path object.
 		public override void Draw(Graphics graphics, Pen pen)
@@ -1165,6 +1645,100 @@ public sealed class GraphicsPath : MarshalByRefObject, ICloneable, IDisposable
 
 	}; // class RectanglePathObject
 
+	// String path objects.
+	private sealed class StringPathObject : PathObject
+	{
+		// Internal state.
+		String 			mString;
+		RectangleF		mRect;
+		PointF			mPoint;
+		StringFormat 	mFormat;
+		Font				mFont;
+
+		// Constructor.
+		public StringPathObject( String s, FontFamily family, int style, float emSize, PointF origin, StringFormat format ) :
+				this( s, family, style, emSize, origin, RectangleF.Empty, format )
+		{
+		}
+
+		// Constructor.
+		public StringPathObject( String s, FontFamily family, int style, float emSize, RectangleF layoutRect, StringFormat format ) :
+				this( s, family, style, emSize, PointF.Empty, layoutRect, format )
+		{
+		}
+		
+		private StringPathObject( String s, FontFamily family, int style, float emSize, PointF origin, RectangleF layoutRect, StringFormat format ) :
+			this( s, new Font( family, emSize, (FontStyle)style ), origin, layoutRect, format )
+		{
+		}
+		
+		private StringPathObject( String s, Font font, PointF origin, RectangleF layoutRect, StringFormat format )
+		{
+			mString		= s;
+			mFont			= font;
+			mPoint		= origin;
+			mRect			= layoutRect;
+			mFormat		= format;
+			mFormat.FormatFlags |= StringFormatFlags.NoClip;
+		}
+		
+		public override bool HasPathPoints { 
+			get {
+				return false;
+			} 
+		}
+
+						
+		protected override PointF [] DoGetPathPoints() {
+			// How should I ????
+			return null;
+		}
+
+		
+		RectangleF GetRect(Graphics graphics ) {
+			RectangleF ret;
+			if( RectangleF.Empty != mRect ) {
+				ret = mRect;
+			}
+			else {
+				SizeF sz = graphics.MeasureString( mString, mFont, mPoint, mFormat );
+				ret = new RectangleF( mPoint, sz );
+			}
+			return ret;
+		}
+		
+		// Draw this path object.
+		public override void Draw(Graphics graphics, Pen pen)
+		{
+			using( SolidBrush b = new SolidBrush(pen.Color) ) {
+				this.Fill( graphics, b, pen, FillMode.Alternate );
+			}
+		}
+
+		// Fill this path object.
+		public override void Fill(Graphics graphics, Brush brush,
+										  Pen penBrush, FillMode fillMode)
+		{
+			RectangleF r = GetRect(graphics);
+			if( null != mMatrix ) {
+				Matrix oldMatrix = graphics.Transform;
+				graphics.Transform = mMatrix;
+				graphics.DrawString(mString, mFont, brush, r, mFormat);
+				graphics.Transform = oldMatrix;
+			}
+			else {
+				graphics.DrawString(mString, mFont, brush, r, mFormat);
+			}
+		}
+
+		// Clone this path object.
+		public override PathObject Clone()
+		{
+			return (PathObject) this.MemberwiseClone();
+		}
+
+	}; // class RectanglePathObject
+
 }; // class GraphicsPath
 
 }; // namespace System.Drawing.Drawing2D
