@@ -77,7 +77,7 @@ public class RectangleConverter : TypeConverter
 
 				// Extract the string and trim it.
 				String str = ((String)value).Trim();
-				if(str == String.Empty)
+				if(str.Length == 0)
 				{
 					return null;
 				}
