@@ -34,6 +34,8 @@ public sealed class ToolkitManager
 	private static IToolkit toolkit;
 	private static IToolkitPrintingSystem printingSystem;
 
+	public ToolkitManager() {}
+
 	// Get or set the active graphical display toolkit.
 	public static IToolkit Toolkit
 			{
@@ -147,12 +149,50 @@ public sealed class ToolkitManager
 				return new Graphics(graphics);
 			}
 
+	// Create a "Graphics" object from an "IToolkitGraphics" handler.
+	public static Graphics CreateGraphics(IToolkitGraphics graphics, Rectangle baseWindow)
+			{
+				if(graphics == null)
+				{
+					throw new ArgumentNullException("graphics");
+				}
+				return new Graphics(graphics, baseWindow);
+			}
+	
+	// Create a "Graphics" object from an "IToolkitGraphics" handler.
+	public static Graphics CreateGraphics(Graphics graphics, Rectangle baseWindow)
+			{
+				if(graphics == null)
+				{
+					throw new ArgumentNullException("graphics");
+				}
+				return new Graphics(graphics, baseWindow);
+			}
+
+	// Create a "Graphics" object from an "IToolkitGraphics" handler.
+	// Start with a clip that has already been set in the underlying IToolkitGraphics.
+	public static Graphics CreateGraphics(IToolkitGraphics graphics, Region clip)
+			{
+				if(graphics == null)
+				{
+					throw new ArgumentNullException("graphics");
+				}
+				Graphics createdGraphics = new Graphics(graphics);
+				createdGraphics.SetClipInternal(clip);
+				return createdGraphics;
+			}
+
 	// Draw a bitmap-based glyph to a "Graphics" object.  "bits" must be
 	// in the form of an xbm bitmap.
 	public static void DrawGlyph(Graphics graphics, int x, int y,
 								 byte[] bits, int bitsWidth, int bitsHeight,
 								 Color color)
 			{
+				if(graphics == null)
+				{
+					throw new ArgumentNullException("graphics", "Argument cannot be null");
+				}
+
 				graphics.DrawGlyph(x, y, bits, bitsWidth, bitsHeight, color);
 			}
 
@@ -215,7 +255,7 @@ public sealed class ToolkitManager
 				}
 
 				// Bail out if no toolkit name specified.
-				if(name == null || name == String.Empty)
+				if(name == null || name.Length == 0)
 				{
 					return null;
 				}
