@@ -76,6 +76,8 @@ namespace System.Windows.Forms
 			SetStyle(ControlStyles.ResizeRedraw, true);
 			textEntry.BorderStyle = BorderStyle.None;
 			textEntry.TabStop = false;
+			textEntry.LostFocus += new EventHandler(textentry_LostFocus);
+			textEntry.GotFocus += new EventHandler(textentry_GotFocus);
 			this.Controls.Add(textEntry);
 		
 			popup = new PopupControl();
@@ -92,7 +94,6 @@ namespace System.Windows.Forms
 			scrollbar.BackColor = SystemColors.Control;
 			scrollbar.ValueChanged+=new EventHandler(scrollbar_ValueChanged);
 			popup.Controls.Add(scrollbar);
-
 		}
 
 		protected override Size DefaultSize
@@ -219,7 +220,8 @@ namespace System.Windows.Forms
 			}
 			set
 			{
-				if (value == droppedDown)
+				if (value == droppedDown ||
+						Items.Count == 0)
 					return;
 				droppedDown = value;
 				if (value)
@@ -233,11 +235,11 @@ namespace System.Windows.Forms
 					buttonState = ButtonState.Pushed;
 					using (Graphics g = CreateGraphics())
 						DrawButton(g);
-					scrollbar.Maximum = items.Count - 1;
+					scrollbar.Maximum = Items.Count - 1;
 					scrollbar.LargeChange = maxDropDownItems;
 					scrollbar.Value = (selectedIndex==-1) ? 0 :  selectedIndex;
-					if (items.Count < MaxDropDownItems)
-						popup.Height = items.Count * actualItemHeight;
+					if (Items.Count < MaxDropDownItems)
+						popup.Height = Items.Count * actualItemHeight;
 					else
 						popup.Height = maxDropDownItems * actualItemHeight;
 					popupMouseItem = selectedIndex;
@@ -540,7 +542,7 @@ namespace System.Windows.Forms
 				
 				for (int i = 0; i < Items.Count; i++)
 				{
-					if (string.Compare(value, items[i].ToString()) == 0)
+					if (string.Compare(value, Items[i].ToString()) == 0)
 					{
 						SelectedIndex = i;
 						return; 
@@ -591,11 +593,6 @@ namespace System.Windows.Forms
 			base.OnEnter (e);
 			if (DropDownStyle == ComboBoxStyle.DropDownList)
 				Invalidate();
-			else
-			{
-				textEntry.Focus();
-				textEntry.SelectAll();
-			}
 		}
 
 		protected override void OnLeave(EventArgs e)
@@ -914,7 +911,7 @@ namespace System.Windows.Forms
 			{
 				int y = 0;
 				g.SetClip(new Rectangle(0, 0, popupDrawWidth, popup.Height));
-				for (int i = scrollbar.Value; i < items.Count; i++)
+				for (int i = FirstDdItem; i < Items.Count; i++)
 				{
 					if (pos == -1 | i == pos)
 					{
@@ -924,12 +921,12 @@ namespace System.Windows.Forms
 							if (popupMouseItem == i)
 							{
 								g.FillRectangle(SystemBrushes.Highlight, layout); 
-								g.DrawString(items[i].ToString(), Font, SystemBrushes.HighlightText, 0, y);
+								g.DrawString(Items[i].ToString(), Font, SystemBrushes.HighlightText, 0, y);
 							}
 							else
 							{
 								g.FillRectangle(backBrush, layout); 
-								g.DrawString(items[i].ToString(), Font, foreBrush, 0, y);
+								g.DrawString(Items[i].ToString(), Font, foreBrush, 0, y);
 							}
 						}
 						else
@@ -952,7 +949,17 @@ namespace System.Windows.Forms
 				}
 			}
 		}
-		
+
+		private void textentry_LostFocus(object sender, EventArgs e)
+		{
+			OnLostFocus(e);
+		}
+
+		private void textentry_GotFocus(object sender, EventArgs e)
+		{
+			OnGotFocus(e);
+		}
+	
 		private void scrollbar_ValueChanged(object sender, EventArgs e)
 		{
 			using (Graphics g = popup.CreateGraphics())
@@ -968,7 +975,9 @@ namespace System.Windows.Forms
 		{
 			if (e.Y > 0 & e.X < popupDrawWidth)
 			{
-				SelectedIndex = ItemFromY(e.Y);
+				int itemFromY = ItemFromY(e.Y);
+				if (itemFromY < Items.Count)
+					SelectedIndex = itemFromY;
 				DroppedDown = false;
 				if (DropDownStyle == ComboBoxStyle.DropDownList)
 					Invalidate();
@@ -1004,12 +1013,32 @@ namespace System.Windows.Forms
 			}
 		}
 
+		// Index of first item in the DrowpDown list
+		private int FirstDdItem
+		{
+			get
+			{
+				if(items.Count <= MaxDropDownItems)
+				{
+					return 0;
+				}
+				else if(scrollbar.Value + MaxDropDownItems < Items.Count)
+				{
+					return scrollbar.Value;
+				}
+				else
+				{
+					return Items.Count - MaxDropDownItems;
+				}
+			}
+		}
+
 		private int ItemFromY(int y)
 		{
 			if (DrawMode != DrawMode.OwnerDrawVariable)
-				return y / actualItemHeight + scrollbar.Value;
+				return y / actualItemHeight + FirstDdItem;
 			int itemY = 0;
-			for (int i = scrollbar.Value; i < items.Count; i++)
+			for (int i = FirstDdItem; i < Items.Count; i++)
 			{
 				itemY += itemHeights[i];
 				if (y < itemY)
