@@ -534,17 +534,17 @@ public class FontDialog : CommonDialog
 
 					// Add the top-level hbox to the dialog and set the size.
 					Controls.Add(hbox);
-					Size size = hbox.RecommendedSize;
-					if(size.Width < 450)
+					Size drawsize = hbox.RecommendedSize;
+					if(drawsize.Width < 450)
 					{
-						size.Width = 450;
+						drawsize.Width = 450;
 					}
-					if(size.Height < 280)
+					if(drawsize.Height < 280)
 					{
-						size.Height = 280;
+						drawsize.Height = 280;
 					}
-					ClientSize = size;
-					MinimumSize = size;
+					ClientSize = drawsize;
+					MinimumSize = drawsize;
 					MinimizeBox = false;
 					ShowInTaskbar = false;
 
@@ -791,7 +791,7 @@ public class FontDialog : CommonDialog
 					int index = nameList.SelectedIndex;
 					if(index >= 0)
 					{
-						name.Text = (nameList.Items[index]).ToString();
+						name.Text = ((IList)nameList.Items)[index].ToString();
 					}
 					else
 					{
@@ -807,7 +807,7 @@ public class FontDialog : CommonDialog
 					int index = sizeList.SelectedIndex;
 					if(index >= 0)
 					{
-						size.Text = (sizeList.Items[index]).ToString();
+						size.Text = ((IList)sizeList.Items)[index].ToString();
 					}
 					SetFont();
 					UpdateSample();
