@@ -26,6 +26,7 @@ namespace System.Windows.Forms
 using System.Drawing;
 using System.Drawing.Toolkit;
 using System.ComponentModel;
+using System.Collections;
 
 public class Form : ContainerControl
 {
@@ -56,6 +57,7 @@ public class Form : ContainerControl
 	private Color transparencyKey;
 	private FormWindowState windowState;
 	internal static Form activeForm;
+	internal static ArrayList activeForms = new ArrayList();
 	private bool showInTaskbar;
 	private bool controlBox;
 	private bool loaded; /* whether we have sent OnLoad or not */
@@ -409,8 +411,10 @@ public class Form : ContainerControl
 					{
 						return;
 					}
+					
 					mdiParent = value;
 					value.mdiClient.Controls.Add(this);
+					Parent = value.mdiClient;
 				}
 			}
 	public bool MinimizeBox
@@ -597,7 +601,7 @@ public class Form : ContainerControl
 				}
 				set
 				{
-					// Brubbel recreate toolkitwindow, if exists
+					// recreate toolkitwindow, if exists
 					if( value != topLevel ) {
 						topLevel = value;
 						if( null != toolkitWindow ) {
@@ -666,86 +670,91 @@ public class Form : ContainerControl
 				}
 			}
 
-	// Create the toolkit window underlying this control.
 	internal override IToolkitWindow CreateToolkitWindow(IToolkitWindow parent)
-			{
-				// When a Form is reparented to a normal container control 
-				// which does work on Win32 unfortunately.
-				if(mdiParent == null && (!TopLevel))
-				{
-					return base.CreateToolkitWindow(parent);
-				}
+	{
+		// When a Form is reparented to a normal container control 
+		// which does work on Win32 unfortunately.
+		if(mdiParent == null && (!TopLevel))
+		{
+			return base.CreateToolkitWindow(parent);
+		}
 
-				CreateParams cp = CreateParams;
+		CreateParams cp = CreateParams;
 
-				// Create the window and set its initial caption.
-				IToolkitTopLevelWindow window;
-				if(mdiParent == null)
-				{
-					window = ToolkitManager.Toolkit.CreateTopLevelWindow
-						(cp.Width - ToolkitDrawSize.Width,
-						 cp.Height - ToolkitDrawSize.Height, this);
-				}
-				else
-				{
-					mdiParent.mdiClient.CreateControl();
-					IToolkitMdiClient mdi =
-						(mdiParent.mdiClient.toolkitWindow as
-							IToolkitMdiClient);
-					window = mdi.CreateChildWindow
-						(cp.X, cp.Y,
-						 cp.Width - ToolkitDrawSize.Width,
-						 cp.Height - ToolkitDrawSize.Height, this);
-				}
-				window.SetTitle(cp.Caption);
-				// Win32 requires this because if the window is maximized, the windows size needs to be set.
-				toolkitWindow = window;
+		// Create the window and set its initial caption.
+		IToolkitTopLevelWindow window;
+		if(mdiParent == null)
+		{
+			// use ControlToolkitManager to create the window thread safe
+			window = ControlToolkitManager.CreateTopLevelWindow( this,
+					cp.Width - ToolkitDrawSize.Width,
+					cp.Height - ToolkitDrawSize.Height);
+		}
+		else
+		{
+			mdiParent.mdiClient.CreateControl();
+			IToolkitMdiClient mdi =
+					(mdiParent.mdiClient.toolkitWindow as
+					IToolkitMdiClient);
+			
+			// use ControlToolkitManager to create the window thread safe
+			window = ControlToolkitManager.CreateMdiChildWindow( this, mdi, 
+					cp.X, cp.Y,
+					cp.Width - ToolkitDrawSize.Width,
+					cp.Height - ToolkitDrawSize.Height);
+		}
+		
+		window.SetTitle(cp.Caption);
+		// Win32 requires this because if the window is maximized, the windows size needs to be set.
+		toolkitWindow = window;
 
-				// Adjust the window hints to match our requirements.
-				SetWindowFlags(window);
-				if(icon != null)
-				{
-					window.SetIcon(icon);
-				}
-				window.SetMaximumSize(maximumSize);
-				window.SetMinimumSize(minimumSize);
-			#if !CONFIG_COMPACT_FORMS
-				if(windowState == FormWindowState.Minimized)
-				{
-					window.Iconify();
-				}
-				else
-			#endif
-				if(windowState == FormWindowState.Maximized)
-				{
-					window.Maximize();
-				}
+		// Adjust the window hints to match our requirements.
+		SetWindowFlags(window);
+		if(icon != null)
+		{
+			window.SetIcon(icon);
+		}
+		window.SetMaximumSize(maximumSize);
+		window.SetMinimumSize(minimumSize);
+
+#if !CONFIG_COMPACT_FORMS
+		if(windowState == FormWindowState.Minimized)
+		{
+			window.Iconify();
+		}
+		else
+#endif
+		if(windowState == FormWindowState.Maximized)
+		{
+			window.Maximize();
+		}
+
+		// Center the window on-screen if necessary.
+		if(formStartPosition == FormStartPosition.CenterScreen)
+		{
+			Size screenSize = ToolkitManager.Toolkit.GetScreenSize();
+			window.MoveResize
+					((screenSize.Width - cp.Width) / 2,
+					(screenSize.Height - cp.Height) / 2,
+					window.Dimensions.Width, window.Dimensions.Height);
+		}
+		else if(formStartPosition == FormStartPosition.Manual)
+		{
+			window.MoveResize
+					(
+					cp.X,
+			cp.Y,
+			window.Dimensions.Width,
+			window.Dimensions.Height
+					);
+		}
+		if(opacity != 1.0)
+		{
+			window.SetOpacity(opacity);
+		}
+		return window;
+}
 
-				// Center the window on-screen if necessary.
-				if(formStartPosition == FormStartPosition.CenterScreen)
-				{
-					Size screenSize = ToolkitManager.Toolkit.GetScreenSize();
-					window.MoveResize
-							((screenSize.Width - cp.Width) / 2,
-							 (screenSize.Height - cp.Height) / 2,
-							 window.Dimensions.Width, window.Dimensions.Height);
-				}
-				else if(formStartPosition == FormStartPosition.Manual)
-				{
-					window.MoveResize
-						(
-							cp.X,
-							cp.Y,
-							window.Dimensions.Width,
-							window.Dimensions.Height
-						);
-				}
-				if(opacity != 1.0)
-				{
-					window.SetOpacity(opacity);
-				}
-				return window;
-			}
 
 	// Determine if this is a top-level control which cannot have parents.
 	internal override bool IsTopLevel
@@ -954,6 +963,7 @@ public class Form : ContainerControl
 
 					// Enter a message loop until the dialog result is set.
 					Application.InnerMessageLoop(this);
+					Dispose();	// must call Dispose, when closing the dialog
 				}
 				finally
 				{
@@ -962,7 +972,7 @@ public class Form : ContainerControl
 					// The form is no longer modal.
 					SetWindowFlag(ToolkitWindowFlags.Modal, false);
 				}
-
+				
 				// Return the dialog result.
 				return dialogResult;
 			}
@@ -1171,6 +1181,37 @@ public class Form : ContainerControl
 	// Dispose of this control.
 	protected override void Dispose(bool disposing)
 			{
+				acceptButton = null;
+				defaultButton = null;
+				cancelButton = null;
+				mdiClient = null;
+				if( null != owner ) {
+					this.RemoveOwnedForm(owner);
+					owner = null;
+				}
+				if( null != ownedForms ) {
+					int iCount = ownedForms.Length;
+					for( int i = iCount-1; i >= 0; i-- ) {
+						if( null != ownedForms[i] ) {
+							ownedForms[i].Dispose();
+						}
+					}
+				}
+
+				if( null != menu ) {
+					menu.ownerForm = null;
+					menu = null;
+				}
+
+				if( null != mergedMenu ) {
+					if( mergedMenu.ownerForm == this || mergedMenu.ownerForm == null ) {
+						mergedMenu.Dispose();
+					}
+					mergedMenu = null;
+				}
+				
+				if( activeForm == this ) activeForm = null;
+
 				base.Dispose(disposing);
 			}
 
@@ -1220,6 +1261,11 @@ public class Form : ContainerControl
 					loaded = true;
 					OnLoad(EventArgs.Empty);
 				}
+				if( menu != null ) {
+					// Workaround for fixing that menu is dispayed correct when form is shown.
+					Height = Height+1;
+					Height = Height-1;
+				}
 			}
 
 	// Emit the "Deactivate" event.
@@ -1246,12 +1292,16 @@ public class Form : ContainerControl
 	// Override the "HandleCreated" event.
 	protected override void OnHandleCreated(EventArgs e)
 			{
+				// we have to add a reference to the form if the form is created and shown without a reference
+				// to avoid the form beeing collected, even it is shown.
+				if( !activeForms.Contains(this) ) activeForms.Add(this);
 				base.OnHandleCreated(e);
 			}
 
 	// Override the "HandleDestroyed" event.
 	protected override void OnHandleDestroyed(EventArgs e)
 			{
+				activeForms.Remove(this);
 				base.OnHandleDestroyed(e);
 			}
 
@@ -1618,10 +1668,13 @@ public class Form : ContainerControl
 	// Close request received from "Control.ToolkitClose".
 	internal override void CloseRequest()
 			{
+				if( IsDisposed ) return;	// irgnore CloseRequest, if form was destroyed
+				
 				CancelEventArgs args = new CancelEventArgs();
 				OnClosing(args);
 				if(!(args.Cancel))
 				{
+					dialogResultIsSet = true;	// must be set here, or Application.InnerMessageLoop won't end
 					OnClosed(EventArgs.Empty);
 					Dispose();
 				}
@@ -1685,7 +1738,6 @@ public class Form : ContainerControl
 
 	}; // class ControlCollection
 
-
 	protected override void OnMouseDown(MouseEventArgs e)
 			{
 				// If the mouse is in the non client area,
