@@ -37,6 +37,7 @@ public class ListBox : ListControl
 	private VScrollBar vertScrollbar;
 	private HScrollBar horizScrollbar;
 	private ObjectCollection internalDataSource;
+	private SolidBrush mtextBrush;
 	private SolidBrush backgroundBrush;
 	private SolidBrush disabledBrush;
 	private int focusedItem = 0;
@@ -379,7 +380,10 @@ public class ListBox : ListControl
 				List[index] = value; 
 				
 				if(this.owner != null && !this.owner.suppressEvents)
+				{
+					this.owner.FireSelectedValueChanged();
 					this.owner.FireSelectedIndexChanged();
+				}
 			}
   		}
 
@@ -391,7 +395,10 @@ public class ListBox : ListControl
 				List[index] = value; 
 
 				if(this.owner != null && !this.owner.suppressEvents)
+				{
+					this.owner.FireSelectedValueChanged();
 					this.owner.FireSelectedIndexChanged();
+				}
 			}
   		}
 
@@ -409,7 +416,10 @@ public class ListBox : ListControl
 			{
 				this.owner.PaintItem(value);
 				if(!this.owner.suppressEvents)
+				{
+					this.owner.FireSelectedValueChanged();
 					this.owner.FireSelectedIndexChanged();
+				}
 			}
 			
  			return result;
@@ -433,7 +443,10 @@ public class ListBox : ListControl
 					this.owner.PaintItem((int) o);
 			
 				if(!this.owner.suppressEvents)
+				{
+					this.owner.FireSelectedValueChanged();
 					this.owner.FireSelectedIndexChanged();
+				}
 			}
  		}
 
@@ -454,7 +467,10 @@ public class ListBox : ListControl
 				this.owner.FlushPaint();
 
 				if(!this.owner.suppressEvents)
+				{
+					this.owner.FireSelectedValueChanged();
 					this.owner.FireSelectedIndexChanged();
+				}
 			}
  		}
 
@@ -476,7 +492,10 @@ public class ListBox : ListControl
  				this.owner.PaintItem((int) value);
 
 				if(!this.owner.suppressEvents)
+				{
+					this.owner.FireSelectedValueChanged();
 					this.owner.FireSelectedIndexChanged();
+				}
 			}
  		}
 		
@@ -488,7 +507,10 @@ public class ListBox : ListControl
  				this.owner.PaintItem((int) value);
 
 				if(!this.owner.suppressEvents)
+				{
+					this.owner.FireSelectedValueChanged();
 					this.owner.FireSelectedIndexChanged();
+				}
 			}
  		}
 
@@ -501,7 +523,10 @@ public class ListBox : ListControl
 				this.owner.PaintItem(value);
 
 				if(!this.owner.suppressEvents)
+				{
+					this.owner.FireSelectedValueChanged();
 					this.owner.FireSelectedIndexChanged();
+				}
 			}
  		}
 
@@ -673,11 +698,13 @@ public class ListBox : ListControl
 		{
 			get
 			{
-				return true;	
+				return true;
 			}
 		}
 	} // class SelectedObjectCollection;
 
+	public const int DefaultItemHeight=13;
+	
 	public const int NoMatches = -1;
 
 	public override Color BackColor
@@ -745,6 +772,10 @@ public class ListBox : ListControl
 		set
 		{
 			base.ForeColor = value;
+			if(this.mtextBrush != null ) {
+				this.mtextBrush.Dispose();
+			} 
+			this.mtextBrush = new SolidBrush( base.ForeColor );
 		}
 	}
 
@@ -776,8 +807,12 @@ public class ListBox : ListControl
 	[TODO]
 	public virtual int ItemHeight
 	{
-		get { return itemHeight; }
-		set { itemHeight = value; }
+		get {
+			return itemHeight; 
+		}
+		set { 
+			itemHeight = value; 
+		}
 	}
 
 	public ListBox.ObjectCollection Items
@@ -802,12 +837,12 @@ public class ListBox : ListControl
 		}
 	}
 
-	[TODO]
+	//[TODO]
 	public int PreferredHeight
 	{
 		get
 		{
-			return 0;
+			return (this.internalDataSource.Count * this.ItemHeight)+ SystemInformation.BorderSize.Height * 4 + 3;
 		}
 	}
 
@@ -853,12 +888,13 @@ public class ListBox : ListControl
 
 			IList data = (IList) base.dataSource;
 			
-			if(value < 0 || value >= data.Count)
+			if(value < -1 || value >= data.Count)
 				throw new ArgumentOutOfRangeException(/* TODO */);
 		
 			if(this.selectionMode == SelectionMode.One)
 				this.selectedIndices.Clear();
-			this.selectedIndices.Add(value);
+			if(value != -1)
+				this.selectedIndices.Add(value);
 		}
 	}
 
@@ -1001,34 +1037,34 @@ public class ListBox : ListControl
 		this.FlushPaint();
 	}
 
-	[TODO]
+	//[TODO]
 	public int FindString( String s )
 	{
-		return -1;
+		return FindStringInternal(s, internalDataSource, -1, false);
 	}
 
-	[TODO]
+	//[TODO]
 	public int FindString( String s, int index )
 	{
-		return -1;
+		return FindStringInternal(s, internalDataSource, index, false);
 	}
 
-	[TODO]
+	//[TODO]
 	public int FindStringExact( String s )
 	{
-		return -1;
+		return FindStringInternal(s, internalDataSource, -1, true);
 	}
 
-	[TODO]
+	//[TODO]
 	public int FindStringExact( String s, int index )
 	{
-		return -1;
+		return FindStringInternal(s, internalDataSource, index, true);
 	}
 
 	[TODO]
 	public int GetItemHeight()
 	{
-		return 0;
+		return this.ItemHeight;
 	}
 
 	public Rectangle GetItemRectangle(int index)
@@ -1195,6 +1231,11 @@ public class ListBox : ListControl
 		base.OnResize( e );
 	}
 	
+	private void FireSelectedValueChanged()
+	{
+		this.OnSelectedValueChanged(new EventArgs());
+	}
+	
 	private void FireSelectedIndexChanged()
 	{
 		this.OnSelectedIndexChanged(new EventArgs());
@@ -1243,6 +1284,7 @@ public class ListBox : ListControl
 		this.internalDataSource = this.CreateItemCollection();
 		base.dataSource         = this.internalDataSource;
 
+		this.mtextBrush          = new SolidBrush(SystemColors.WindowText); 
 		this.backgroundBrush     = new SolidBrush(SystemColors.Window);
 		this.disabledBrush       = new SolidBrush(SystemColors.Control);
 		this.selectedIndices     = new SelectedIndexCollection(this);
@@ -1260,7 +1302,9 @@ public class ListBox : ListControl
 			+= new ScrollEventHandler(this.OnHScroll);
 		this.horizScrollbar.ValueChanged 
 			+= new EventHandler(this.OnHValueChanged);
+		
 		BorderStyle = BorderStyle.Fixed3D;
+		SetStyle(ControlStyles.UserPaint | ControlStyles.StandardClick, false);
 	}
 	
 	private void PositionControls()
@@ -1289,6 +1333,8 @@ public class ListBox : ListControl
 	{
 		get
 		{
+			if( this.NumItemsPerColumn <= 0 ) return 0; // avoid div by zero
+			
 			IList data = (IList) base.dataSource;
 			double dColumnsNeeded = (double) data.Count / this.NumItemsPerColumn;
 
@@ -1456,8 +1502,24 @@ public class ListBox : ListControl
 		
 		// Set up the vertical scrollbar
 		IList data = (IList) base.dataSource;
-		if(data.Count == 0)
+		
+		int iDataCount = data.Count;
+		// cleanup all removed items in selction list
+		
+		this.suppressEvents = true;
+		
+		for( int i = 0; i < this.selectedIndices.Count; i++ ) {
+			if( this.selectedIndices[i] >= iDataCount ) {
+				this.selectedIndices.RemoveAt(i);
+			} 
+		}
+		
+		this.suppressEvents = false;
+		
+		if(iDataCount == 0) {
 			this.vertScrollbar.Visible = false;
+			// this.ClearSelected();
+		}
 		else
 		{
 			this.vertScrollbar.Maximum = data.Count;
@@ -1510,6 +1572,12 @@ public class ListBox : ListControl
 		{
 			return topIndex;
 		}
+		set
+		{
+			this.topIndex = value;
+			this.vertScrollbar.Value = this.topIndex;
+			RedrawList();
+		}
 	}
 	
 	// How many columns can be displayed.
@@ -1517,6 +1585,8 @@ public class ListBox : ListControl
 	{
 		get
 		{
+			if( this.columnWidth <= 0 ) return 0; // avoid div by zero
+			
 			int vscrollWidth = 0;
 			if(this.vertScrollbar.Visible)
 				vscrollWidth = this.vertScrollbar.Width;
@@ -1564,7 +1634,12 @@ public class ListBox : ListControl
 	{
 		get
 		{
-			return (this.Height) / this.ItemHeight;
+			if( this.ItemHeight > 0 ) { // avoid div by zero
+				return (this.Height) / this.ItemHeight;
+			}
+			else {
+				return (this.Height) / DefaultItemHeight;
+			}
 		}
 	}
 	
@@ -1577,6 +1652,7 @@ public class ListBox : ListControl
 		
 			if(this.multiColumn)
 			{
+				if( this.NumItemsPerColumn <= 0 ) return new Rectangle(-1, -1, -1, -1); // avoid div by zero
 				int col = screenIndex / this.NumItemsPerColumn;
 				int row = screenIndex - this.NumItemsPerColumn * col;
 				
@@ -1636,6 +1712,12 @@ public class ListBox : ListControl
 		}
 	}
 	
+	protected internal virtual void PaintItem(Graphics g, int index, string text, Font font, Brush textBrush,
+		Rectangle itemRect, StringFormat format)
+	{
+		g.DrawString(text, this.Font, textBrush, itemRect, format);
+	}
+
 	private void PaintItem(Graphics g, int dataIndex)
 	{
 		if(this.suppressDraw)
@@ -1651,7 +1733,7 @@ public class ListBox : ListControl
 		Rectangle realItemRect = this.GetItemRect(dataIndex);
 		IList data = (IList) base.dataSource;
 
-		Brush textBrush = SystemBrushes.WindowText;
+		Brush textBrush = this.mtextBrush;
 		Brush itemBackgroundBrush = this.backgroundBrush;
 
 		// Is this item selected?
@@ -1698,7 +1780,7 @@ public class ListBox : ListControl
 			}
 			
 			// Paint the text
-			g.DrawString(s, this.Font, textBrush, itemRect, new StringFormat());
+			PaintItem(g, dataIndex, s, this.Font, textBrush, itemRect, new StringFormat());
 			
 			// Does this item have the focus rectangle?
 			if(this.FocusedItem == dataIndex)
@@ -1713,7 +1795,7 @@ public class ListBox : ListControl
 				else
 				{
 					backColor = this.BackColor;
-					foreColor = SystemColors.WindowText;
+					foreColor = this.ForeColor;
 				}
 				
 				realItemRect.Height--;
@@ -1844,6 +1926,7 @@ public class ListBox : ListControl
 			this.FlushPaint();
 
 			this.suppressEvents = false;
+			this.FireSelectedValueChanged();
 			this.FireSelectedIndexChanged();
 		}	
 	}
@@ -1878,6 +1961,7 @@ public class ListBox : ListControl
 					this.suppressEvents = true;
 					this.MoveSelection(dataInd);
 					this.suppressEvents = false;
+					this.FireSelectedValueChanged();
 					this.FireSelectedIndexChanged();
 					break;
 				
@@ -1927,6 +2011,7 @@ public class ListBox : ListControl
 						this.selectedIndices.Clear();
 						this.selectedIndices.Add(dataInd);
 						this.suppressEvents = false;
+						this.FireSelectedValueChanged();
 						this.FireSelectedIndexChanged();
 					}
 					
@@ -1988,6 +2073,7 @@ public class ListBox : ListControl
 					this.suppressEvents = true;
 					this.MoveSelection(dataInd);
 					this.suppressEvents = false;
+					this.FireSelectedValueChanged();
 					this.FireSelectedIndexChanged();
 					break;
 					
@@ -2034,6 +2120,7 @@ public class ListBox : ListControl
 			// In a multi-column listbox, only scroll by entire columns.
 			if(this.multiColumn)
 			{
+				if( this.NumItemsPerColumn <= 0 ) return; // avoid div by zero
 				col = this.FocusedItem / this.NumItemsPerColumn;
 			
 				if(this.FocusedItem >= this.topIndex + this.NumItemsVisible)
@@ -2060,6 +2147,16 @@ public class ListBox : ListControl
 		}
 	}
 
+	protected override bool ProcessDialogKey(Keys keyData)
+	{
+		KeyEventArgs evArgs = new KeyEventArgs( keyData ); 
+		this.OnKeyDown( evArgs  );
+		if( !evArgs.Handled ) {
+			return base.ProcessDialogKey(keyData);
+		}
+		return true;
+	}
+	
 	protected override void OnKeyDown(KeyEventArgs e)
 	{
 		bool multiEx = (this.selectionMode == SelectionMode.MultiExtended);
@@ -2068,18 +2165,20 @@ public class ListBox : ListControl
 		int maxInd = data.Count - 1;
 
 		this.DelayPaint();
-
+		
 		switch(e.KeyCode)
 		{
 			case Keys.Up:
 			case Keys.Left:
 				if(this.multiColumn && e.KeyCode == Keys.Left)
 				{
-					int col = this.FocusedItem / this.NumItemsPerColumn;
-					int row = this.FocusedItem - this.NumItemsPerColumn * col;
-					col--;
-					this.FocusedItem = Math_Max(row,
-						row + col * this.NumItemsPerColumn);
+					if( this.NumItemsPerColumn > 0 ) { // avoid div by zero
+						int col = this.FocusedItem / this.NumItemsPerColumn;
+						int row = this.FocusedItem - this.NumItemsPerColumn * col;
+						col--;
+						this.FocusedItem = Math_Max(row,
+							row + col * this.NumItemsPerColumn);
+					}
 				}
 				else if(this.FocusedItem > 0)
 					this.FocusedItem--;
@@ -2094,16 +2193,18 @@ public class ListBox : ListControl
 			case Keys.Right:
 				if(this.multiColumn && e.KeyCode == Keys.Right)
 				{
-					int col = this.FocusedItem / this.NumItemsPerColumn;
-					int row = this.FocusedItem - this.NumItemsPerColumn * col;
-					col++;
-					if(col < this.NumDataColumns)
-					{
-						this.FocusedItem = Math_Min(maxInd,
-							row + col * this.NumItemsPerColumn);
+					if( this.NumItemsPerColumn > 0 ) { // avoid div by zero
+						int col = this.FocusedItem / this.NumItemsPerColumn;
+						int row = this.FocusedItem - this.NumItemsPerColumn * col;
+						col++;
+						if(col < this.NumDataColumns)
+						{
+							this.FocusedItem = Math_Min(maxInd,
+								row + col * this.NumItemsPerColumn);
+						}
 					}
 				}
-				else if(this.FocusedItem < maxInd)
+				else if(this.FocusedItem < maxInd) 
 					this.FocusedItem++;
 					
 				if(multiEx && shift)
@@ -2168,6 +2269,7 @@ public class ListBox : ListControl
 				this.suppressEvents = true;	
 				this.MoveSelection(this.FocusedItem);
 				this.suppressEvents = false;
+				this.FireSelectedValueChanged();
 				this.FireSelectedIndexChanged();
 			}
 			else if(multiEx && !shift)
@@ -2176,6 +2278,7 @@ public class ListBox : ListControl
 				this.selectedIndices.Clear();
 				this.selectedIndices.Add(this.FocusedItem);
 				this.suppressEvents = false;
+				this.FireSelectedValueChanged();
 				this.FireSelectedIndexChanged();
 			}
 		}
@@ -2194,6 +2297,8 @@ public class ListBox : ListControl
 	
 		if(listArea.Contains(x, y))
 		{
+			if( this.ItemHeight <= 0 ) return -1; // avoid div by zero
+
 			y -= listArea.Top;
 			int row = y / this.ItemHeight;
 			if(this.multiColumn)
@@ -2202,6 +2307,7 @@ public class ListBox : ListControl
 					return -1;
 					
 				x -= listArea.Left;
+				if( this.columnWidth <= 0 ) return -1; // avoid div by zero
 				int col = x / this.columnWidth;
 				int ind = row + col * this.NumItemsPerColumn;
 				return ind;
