@@ -68,7 +68,6 @@ public abstract class ScrollBar : Control
 		idleTimer.Tick += new EventHandler(idleTimer_Tick);
 		idleTimer.Interval = 1;
 		BackColor = SystemColors.ScrollBar;
-		Cursor = Cursors.Default;
 	}
 
 	// Properties
@@ -426,10 +425,10 @@ public abstract class ScrollBar : Control
 		{
 	
 			
-			track  = new Rectangle(0, s.Width * 2/3 + 1, s.Width, s.Height - (s.Width * 4/3) - 1);
+			track  = new Rectangle(0, s.Width * 2/3, s.Width, s.Height - (s.Width * 4/3));
 		
-			decrement = new Rectangle(0, 0, s.Width, s.Width * 2/3 + 1);
-			increment = new Rectangle(0, s.Width * 2/3 + trackHeight, s.Width, s.Width * 2/3 );
+			decrement = new Rectangle(0, 0, s.Width, s.Width * 2/3);
+			increment = new Rectangle(0, s.Width * 2/3 + trackHeight, s.Width, (s.Width) * 2/3);
 		}
 		else
 		{
