@@ -79,6 +79,8 @@ public class TextBox : TextBoxBase
 	private bool showVScrollBar;
 	private bool showHScrollBar;
 
+	private int graphicsOffset;
+
 	[TODO]
 	public TextBox()
 	{
@@ -87,12 +89,6 @@ public class TextBox : TextBoxBase
 		// TextBox API is maintained.
 		// Note: except for KeyPress events, for which the 
 		// hooked up calls get priority over TextBox class.
-		MouseDown += new MouseEventHandler(HandleMouseDown);
-		MouseMove += new MouseEventHandler(HandleMouseMove);
-		DoubleClick += new EventHandler(HandleDoubleClick);
-		Paint += new PaintEventHandler(HandlePaint);
-		MultilineChanged +=new EventHandler(HandleMultilineChanged);
-		WordWrapChanged +=new EventHandler(HandleWordWrapChanged);
 		
 #if CONFIG_COMPONENT_MODEL
 		this.DataBindings.CollectionChanged +=new CollectionChangeEventHandler(HandleDataBindingCollectionChanged);
@@ -115,10 +111,20 @@ public class TextBox : TextBoxBase
 		SetStyle(ControlStyles.DoubleBuffer | ControlStyles.AllPaintingInWmPaint | ControlStyles.UserPaint, true);
 	
 		// we need this or no text is displayed after first start
-		this.CreateHandle();
+		// this.CreateHandle(); // do we really need this ?????
+		
 	}
 
-	// Gets or sets a value indicating whether pressing ENTER in a multiline TextBox control creates a new line of text in the control or activates the default button for the form.
+	public override Cursor Cursor
+	{
+		set
+		{
+			if( value == null ) base.Cursor = Cursors.IBeam;
+			else                base.Cursor = value;
+		}
+	}
+	
+// Gets or sets a value indicating whether pressing ENTER in a multiline TextBox control creates a new line of text in the control or activates the default button for the form.
 	public bool AcceptsReturn
 	{
 		get
@@ -216,6 +222,7 @@ public class TextBox : TextBoxBase
 		}
 		else if (vScrollBar != null)
 		{
+			vScrollBar.ValueChanged-=new EventHandler(vScrollBar_ValueChanged);
 			Controls.Remove(vScrollBar);
 			vScrollBar = null;
 		}
@@ -235,6 +242,7 @@ public class TextBox : TextBoxBase
 		}
 		else if (hScrollBar != null)
 		{
+			hScrollBar.ValueChanged-=new EventHandler(hScrollBar_ValueChanged);
 			Controls.Remove(hScrollBar);
 			hScrollBar = null;
 		}
@@ -372,38 +380,49 @@ public class TextBox : TextBoxBase
 		}
 		set
 		{
-			// Change all text endings of CR or LF into CRLF
-			System.Text.StringBuilder sb = new System.Text.StringBuilder
-				(value != null ? value.Length : 0);
-			if (value != null && value.Length > 0)
-			{
-				char cPrevious = (char)0;
-				for (int i = 0; i < value.Length; i++)
+			if( base.Text == value ) {	// check if text has changed
+				return;
+			}
+			
+			if( value.IndexOfAny( new char [] { '\n', '\r' } ) >= 0 ) {	// check if chars are in use, before use of StringBuilder (performance)
+			
+				// Change all text endings of CR or LF into CRLF
+				System.Text.StringBuilder sb = new System.Text.StringBuilder
+					(value != null ? value.Length : 0);
+				if (value != null && value.Length > 0)
 				{
-					char c = value[i];
-					bool isNext = (i < value.Length - 1);
-					char cNext = (char)0;
-					if (isNext)
+					char cPrevious = (char)0;
+					for (int i = 0; i < value.Length; i++)
 					{
-						cNext = value[i+1];
+						char c = value[i];
+						bool isNext = (i < value.Length - 1);
+						char cNext = (char)0;
+						if (isNext)
+						{
+							cNext = value[i+1];
+						}
+						if ((!isNext || cNext != '\n') && c == '\r')
+						{
+							sb.Append("\r\n");
+						}
+						else if (c=='\n' && cPrevious != '\r')
+						{
+							sb.Append("\r\n");
+						}
+						else
+						{
+							sb.Append(c);
+						}
+						cPrevious = c;
 					}
-					if ((!isNext || cNext != '\n') && c == '\r')
-					{
-						sb.Append("\r\n");
-					}
-					else if (c=='\n' && cPrevious != '\r')
-					{
-						sb.Append("\r\n");
-					}
-					else
-					{
-						sb.Append(c);
-					}
-					cPrevious = c;
 				}
+				
+				SetTextActual( sb.ToString());
+			}
+			else {
+				SetTextActual( value );
 			}
 			
-			SetTextActual( sb.ToString());
 			// Set the position to the end
 			SelectInternal(Text.Length, 0);
 			if (!inTextChangedEvent)
@@ -575,7 +594,10 @@ public class TextBox : TextBoxBase
 		InvalidateDirty();
 		mouseDown = false;
 		// We dont need to update any selection
-		selectedRegion = null;
+		if( null != selectedRegion ) {
+			selectedRegion.Dispose();
+			selectedRegion = null;
+		}
 	}
 
 	protected override void OnFontChanged(EventArgs e)
@@ -662,32 +684,33 @@ public class TextBox : TextBoxBase
 
 	}
 
-	// Handle "MouseDown" events for the text box.
-	private void HandleMouseDown(Object sender, MouseEventArgs e)
+	protected override void OnMouseDown(MouseEventArgs e)
 	{
+		base.OnMouseDown(e);
 		ProcessMouse(e);
 	}
-
-	// Handle "MouseMove" events for the text box.
-	private void HandleMouseMove(Object sender, MouseEventArgs e)
+	
+	protected override void OnMouseMove(MouseEventArgs e)
 	{
+		base.OnMouseMove(e);
 		ProcessMouse(e);
 	}
-
-	// Handle "DoubleClick" events for the text box.
-	private void HandleDoubleClick(Object sender, EventArgs e)
+	
+	protected override void OnDoubleClick(EventArgs e)
 	{
+		base.OnDoubleClick(e);
 		Point pt = PointToClient(MousePosition);
 		ProcessMouse(new MouseEventArgs(MouseButtons.Left, 2, pt.X, pt.Y, 0));
 	}
-
+	
 	// Handle "Paint" events for the text box.
 	// In our implementation NO painting happens outside of the paint event. This might change because it might not update fast enough
-	private void HandlePaint(Object sender, PaintEventArgs e)
+	protected override void OnPaint(PaintEventArgs e)
 	{
+		base.OnPaint(e);
 		Redraw(e.Graphics);
 	}
-
+	
 	// Redraw a specific portion of the textbox
 	private void Redraw(Graphics g)
 	{
@@ -755,8 +778,9 @@ public class TextBox : TextBoxBase
 	}
 
 	// Handle the event when multiline is changed.
-	private void HandleMultilineChanged(object sender, EventArgs e)
+	protected override void OnMultilineChanged(EventArgs e)
 	{
+		base.OnMultilineChanged(e);
 		layout = null;
 		// Set back the actual chosen height
 		// Will cause LayoutFromText to be called
@@ -831,6 +855,8 @@ public class TextBox : TextBoxBase
 		// We need to do this here so TextDrawArea returns correct values.
 		callSetScrollBarPositions = SetupScrollBars();
 
+		graphicsOffset = g.MeasureCharacters("A", Font, new Rectangle(0,0, 100, 100), new StringFormat())[0].X;
+
 		// We leave 1 pixel on the left and right for the caret
 		// Multiline textboxes are infinite in the y direction
 		// non multiline are infinite in the x direction and we scroll when needed
@@ -842,7 +868,7 @@ public class TextBox : TextBoxBase
 		}
 		else
 		{
-			measureBounds = new Rectangle(1, yLine - 1, maxXY, TextDrawArea.Height - (yLine - 1));
+			measureBounds = new Rectangle(1 - graphicsOffset, yLine - 1, maxXY + graphicsOffset - 1, TextDrawArea.Height - (yLine - 1));
 		}
 
 		string measureText = newText.Substring(posLine);
@@ -972,7 +998,7 @@ public class TextBox : TextBoxBase
 		}
 		if (caretBounds.Left- XViewOffset < TextDrawArea.Left)
 		{
-			XViewOffset = caretBounds.Left;
+			XViewOffset = caretBounds.Left - 1;
 		}
 		else if (caretBounds.Right - XViewOffset > TextDrawArea.Right)
 		{
@@ -995,7 +1021,7 @@ public class TextBox : TextBoxBase
 			}
 			else if (textAlign == HorizontalAlignment.Right)
 			{
-				XViewOffset = maxXY - TextDrawArea.Width + 2;
+				XViewOffset = maxXY - TextDrawArea.Width - graphicsOffset + 1;
 			}
 		}
 		else
@@ -1078,14 +1104,12 @@ public class TextBox : TextBoxBase
 		InvalidateDirty();
 	}
 
-	
 	// Called to change the text. Sets the update to whats needed to but doesnt change the selection point or caret
 	private void SetTextActual( string text)
 	{
-		if (!IsHandleCreated)
-		{
-			(this as Control).text = text;
-			return;
+		if( !IsHandleCreated ) {
+			// create handle here to be sure that LayoutInfo will be set correct.
+			this.CreateHandle();
 		}
 		// Layout the new text. Compare with old layout, Creating a region for areas that must be updated.
 		bool prevLayout = layout != null;
@@ -1101,31 +1125,42 @@ public class TextBox : TextBoxBase
 		SetScrollBarPositions();
 		if (prevLayout)
 		{
-			Region update = new Region(RectangleF.Empty);
-			int len = text.Length;
-			if (oldText.Length > len)
-			{
-				len = oldText.Length;
+			try {
+				Region update = new Region(RectangleF.Empty);
+				int oldLen = oldText.Length;
+				int newLen = text.Length;
+				int len    = newLen;
+						
+				if (oldLen > len)
+				{
+					len = oldLen;
+				}
+				for (int i=0;i < len;i++)
+				{
+					if (i >= oldLen)
+					{
+						if( i < newLen ) update.Union( layout.Items[i].bounds);
+					}
+					else if (i >= newLen )
+					{
+						if( i < oldLen ) update.Union( oldLayout.Items[i].bounds);
+					}
+					else if ( (i < oldLen && i < newLen) && (Text[i] != oldText[i] || oldLayout.Items[i].bounds != layout.Items[i].bounds ) )
+					{
+						if( i < newLen ) {
+							update.Union( layout.Items[i].bounds);
+						}
+						if( i < oldLen ) {
+							update.Union( oldLayout.Items[i].bounds);
+						}
+					}
+				}
+				// Get the offset of the TextDrawArea
+				update.Translate( - XViewOffset, - YViewOffset);
+				AddUpdate(update);
 			}
-			for (int i=0;i < len;i++)
-			{
-				if (i >= oldText.Length)
-				{
-					update.Union( layout.Items[i].bounds);
-				}
-				else if (i >= text.Length)
-				{
-					update.Union( oldLayout.Items[i].bounds);
-				}
-				else if (Text[i] != oldText[i] || oldLayout.Items[i].bounds != layout.Items[i].bounds)
-				{
-					update.Union( layout.Items[i].bounds);
-					update.Union( oldLayout.Items[i].bounds);
-				}
+			catch {	// ignore exceptions here, because in some cases this could happen
 			}
-			// Get the offset of the TextDrawArea
-			update.Translate( - XViewOffset, - YViewOffset);
-			AddUpdate(update);
 		}
 	}
 
@@ -1362,11 +1397,19 @@ public class TextBox : TextBoxBase
 		{
 			height = ClientToBounds(Size.Empty).Height + Font.Height + 1;
 		}
+		
+		Rectangle oldRec = Bounds;
+		
+		if( oldRec.X == x && oldRec.Y == y && oldRec.Width == width && oldRec.Height == height ) { // no update needed if same bounds
+			return;
+		}
+		
 		base.SetBoundsCore (x, y, width, height, specified);
 		if (!IsHandleCreated)
 		{
 			return;
 		}
+		
 		// If the height or width changes then relayout the text
 		if ((specified & BoundsSpecified.Height) != 0 | (specified & BoundsSpecified.Width) != 0)
 		{
@@ -1392,7 +1435,7 @@ public class TextBox : TextBoxBase
 		int x = 0;
 		int y = int.MinValue;
 		int i;
-		using (Brush foreBrush = new SolidBrush(ForeColor), selectedForeBrush = new SolidBrush(SystemColors.HighlightText))
+		using (SolidBrush foreBrush = new SolidBrush(ForeColor), selectedForeBrush = new SolidBrush(SystemColors.HighlightText))
 		{
 			int cleanI = 0;
 			for (i=0; i < text.Length;i++) 
@@ -1420,7 +1463,7 @@ public class TextBox : TextBoxBase
 					// Setup the first values.
 					if (y == int.MinValue)
 					{
-						x = bounds.X;
+						x = bounds.X - graphicsOffset;
 						y = bounds.Y;
 						prevFore = fore;
 					}
@@ -1451,17 +1494,20 @@ public class TextBox : TextBoxBase
 							{
 								lineText = text.Substring(lineStart, cleanI - lineStart + 1);
 							}
+							
 							if (Enabled)
 							{
 								g.DrawString(lineText, font, prevFore, new Point (x, y));
 							}
 							else
 							{
-								ControlPaint.DrawStringDisabled(g, lineText, font, BackColor, new Rectangle(x, y, bounds.Width, bounds.Height), StringFormat.GenericDefault);
+								if(BackColor != Color.Gray)
+									foreBrush.Color = Color.Gray;
+								g.DrawString(lineText, Font, foreBrush, new Point (x, y));
 							}
 						}
 						lineStart = i;
-						x = bounds.X;
+						x = bounds.X - graphicsOffset;
 						y = bounds.Y;
 						prevFore = fore;
 					}
@@ -1488,7 +1534,9 @@ public class TextBox : TextBoxBase
 				}
 				else
 				{
-					ControlPaint.DrawStringDisabled(g, lineText, font, BackColor, new Rectangle(x, y, Bounds.Width, Bounds.Height), StringFormat.GenericDefault);
+					if(BackColor != Color.Gray)
+						foreBrush.Color = Color.Gray;
+					g.DrawString(lineText, Font, foreBrush, new Point (x, y));
 				}
 			}
 		}
@@ -1987,11 +2035,10 @@ public class TextBox : TextBoxBase
 		inTextChangedEvent = false;
 	}
 
-	private void HandleWordWrapChanged(object sender, EventArgs e)
-	{
+	internal override protected void OnWordWrapChanged( EventArgs e ) {
 		layout = null;
 	}
-
+	
 
 	private Rectangle TextDrawArea
 	{
@@ -2061,16 +2108,29 @@ public class TextBox : TextBoxBase
 			invalidateRegion.Dispose();
 			invalidateRegion = null;
 		}
+		if( selectedRegion != null ) {
+			selectedRegion.Dispose();
+			selectedRegion = null;
+		}
 		if (vScrollBar != null)
 		{
+			vScrollBar.ValueChanged-=new EventHandler(vScrollBar_ValueChanged);
 			vScrollBar.Dispose();
 			vScrollBar = null;
 		}
 		if (hScrollBar != null)
 		{
+			hScrollBar.ValueChanged-=new EventHandler(hScrollBar_ValueChanged);
 			hScrollBar.Dispose();
 			hScrollBar = null;
 		}
+		
+		// remove event handler
+		
+#if CONFIG_COMPONENT_MODEL
+		this.DataBindings.CollectionChanged -=new CollectionChangeEventHandler(HandleDataBindingCollectionChanged);
+#endif
+		
 		base.Dispose (disposing);
 	}
 
