@@ -65,7 +65,8 @@ namespace System.Windows.Forms
 		internal int xOffset = 0;
 		const int xPadding = 2;
 		int xScrollValueBeforeEdit = 0;
- 
+		ClickEvent clickEvent;
+
 		public event TreeViewEventHandler AfterCheck
 		{
 			add
@@ -483,11 +484,38 @@ namespace System.Windows.Forms
 							// Draw the node text.
 							if (nodes.currentNode  == selectedNode && (Focused || !hideSelection))
 							{
-								// TODO: FullRowSelect
-								g.FillRectangle(SystemBrushes.Highlight, bounds);
+								Rectangle r;
+
+								// **TODO**
+								// Running into a chicken and egg issue here where if
+								// we fill the rect here, we're overdrawing the
+								// checkBoxes or ExpanderMarker of above
+
+								// Draw FullRowSelect if we qualify
+								if(fullRowSelect && !showLines)
+								{
+									int left = 1;
+//									if(nodes.currentNode)
+//									{
+//										left = 10;
+//									}
+									g.FillRectangle(SystemBrushes.Highlight, new Rectangle(left, bounds.Y, drawableWidth - 1, bounds.Height));
+								}
+								else
+								{
+									g.FillRectangle(SystemBrushes.Highlight, bounds);
+								}
 								g.DrawString(nodes.currentNode.Text, Font, SystemBrushes.HighlightText, bounds, format);
+
 								// Draw the focus rectangle.
-								Rectangle r = new Rectangle(bounds.X - 1, bounds.Y - 1, bounds.Width + 1, bounds.Height + 1);
+								if(FullRowSelect && !ShowLines)
+								{
+									r = new Rectangle(0, bounds.Y - 1, drawableWidth, bounds.Height + 1);
+								}
+								else
+								{
+									r = new Rectangle(bounds.X - 1, bounds.Y - 1, bounds.Width + 1, bounds.Height + 1);
+								}
 								ControlPaint.DrawFocusRectangle(g, r);
 							}
 							else
@@ -549,12 +577,10 @@ namespace System.Windows.Forms
 			int lineTopEnd = midY;
 			if (node.Nodes.Count > 0 && showPlusMinus)
 			{
-				GraphicsPath path = new GraphicsPath();
-				path.AddRectangle(new Rectangle(midX - 4, midY - 4, 8, 8));
-				path.AddLine(midX - 2, midY, midX + 2, midY);
+				g.DrawRectangle(SystemPens.ControlText, midX - 4, midY - 4, 8, 8 );
+				g.DrawLine(SystemPens.ControlText, midX - 2, midY, midX + 2, midY);
 				if (!node.IsExpanded)
-					path.AddLine(midX, midY - 2, midX, midY + 2);
-				g.DrawPath(SystemPens.ControlText, path);
+					g.DrawLine(SystemPens.ControlText, midX, midY - 2, midX, midY + 2);
 				lineRightStart += 6;
 				lineTopEnd -= 6;
 			}
@@ -1069,7 +1095,7 @@ namespace System.Windows.Forms
 			}
 		}
 		
-		protected virtual void OnAfterExpand(TreeViewEventArgs e)
+		protected internal virtual void OnAfterExpand(TreeViewEventArgs e)
 		{
 			TreeViewEventHandler handler = (TreeViewEventHandler)GetHandler(EventId.AfterExpand);
 			if (handler != null)
@@ -1098,7 +1124,7 @@ namespace System.Windows.Forms
 
 		protected internal virtual void OnBeforeCheck(TreeViewCancelEventArgs e)
 		{
-			EventHandler handler = GetHandler(EventId.BeforeCheck) as EventHandler;
+			TreeViewCancelEventHandler handler = (TreeViewCancelEventHandler)GetHandler(EventId.BeforeCheck);
 			if (handler != null)
 			{
 				handler(this,e);
@@ -1107,16 +1133,16 @@ namespace System.Windows.Forms
 
 		protected internal virtual void OnBeforeCollapse(TreeViewCancelEventArgs e)
 		{
-			EventHandler handler = GetHandler(EventId.BeforeCollapse) as EventHandler;
+			TreeViewCancelEventHandler handler = (TreeViewCancelEventHandler)GetHandler(EventId.BeforeCollapse);
 			if (handler != null)
 			{
 				handler(this,e);
 			}
 		}
 
-		protected virtual void OnBeforeExpand(TreeViewCancelEventArgs e)
+		protected internal virtual void OnBeforeExpand(TreeViewCancelEventArgs e)
 		{
-			EventHandler handler = GetHandler(EventId.BeforeExpand) as EventHandler;
+			TreeViewCancelEventHandler handler = (TreeViewCancelEventHandler)GetHandler(EventId.BeforeExpand);
 			if (handler != null)
 			{
 				handler(this,e);
@@ -1125,7 +1151,7 @@ namespace System.Windows.Forms
 
 		protected virtual void OnBeforeLabelEdit(NodeLabelEditEventArgs e)
 		{
-			EventHandler handler = GetHandler(EventId.BeforeLabelEdit) as EventHandler;
+			NodeLabelEditEventHandler handler = (NodeLabelEditEventHandler)GetHandler(EventId.BeforeLabelEdit);
 			if (handler != null)
 			{
 				handler(this,e);
@@ -1134,7 +1160,7 @@ namespace System.Windows.Forms
 
 		protected virtual void OnBeforeSelect(TreeViewCancelEventArgs e)
 		{
-			EventHandler handler = GetHandler(EventId.BeforeSelect) as EventHandler;
+			TreeViewCancelEventHandler handler = (TreeViewCancelEventHandler)GetHandler(EventId.BeforeSelect);
 			if (handler != null)
 			{
 				handler(this,e);
@@ -1162,7 +1188,7 @@ namespace System.Windows.Forms
 
 		protected virtual void OnItemDrag(ItemDragEventArgs e)
 		{
-			EventHandler handler = GetHandler(EventId.ItemDrag) as EventHandler;
+			ItemDragEventHandler handler = (ItemDragEventHandler)GetHandler(EventId.ItemDrag);
 			if (handler != null)
 			{
 				handler(this,e);
@@ -1249,7 +1275,7 @@ namespace System.Windows.Forms
 			}
 			else
 			{
-				ProcessClick(e.X, e.Y, true);
+				ProcessClick(e.X, e.Y, true, ClickEvent.None);
 			}
 			base.OnMouseDown (e);
 		}
@@ -1271,12 +1297,12 @@ namespace System.Windows.Forms
 		// Non Microsoft member.
 		protected override void OnMouseUp(MouseEventArgs e)
 		{
-			ProcessClick(e.X, e.Y, (e.Button == MouseButtons.Right));
+			ProcessClick(e.X, e.Y, (e.Button == MouseButtons.Right), clickEvent);
 
 			base.OnMouseUp (e);
 		}
 
-		void ProcessClick(int x, int y, bool rightMouse)
+		void ProcessClick(int x, int y, bool rightMouse, ClickEvent clickEvent)
 		{
 			int nodeFromTop = -1;
 			int height = ItemHeight;
@@ -1295,9 +1321,18 @@ namespace System.Windows.Forms
 					{
 						// Check if the y matches this node.
 						if (y < height * (nodeFromTop + 1))
-						{
+						{							
 							bool allowEdit = false;
 							bool allowSelect = true;
+							// Raise click or double click when called from MouseUp
+							if(clickEvent == ClickEvent.Click)
+							{
+								OnClick(EventArgs.Empty);
+							}
+							else if(clickEvent == ClickEvent.DoubleClick)
+							{
+								OnDoubleClick(EventArgs.Empty);
+							}
 							// Clicking the image can be used to select.
 							if (imageList == null || !GetImageBounds(nodeFromTop, nodes.level).Contains(x, y))
 							{
@@ -1826,6 +1861,8 @@ namespace System.Windows.Forms
 			{
 				OnMouseDown(new MouseEventArgs(buttons, clicks, x, y, delta));
 			}
+			// Set event to be raised on mouse up
+			clickEvent = (clicks == 1) ? ClickEvent.Click : ClickEvent.DoubleClick;
 		}
 
 		public TreeNode TopNode 
@@ -1982,6 +2019,13 @@ namespace System.Windows.Forms
 				}
 			}
 		}
+		
+		internal enum ClickEvent
+		{
+			Click,
+			DoubleClick,
+			None
+		}
 	}
 
 }
