@@ -27,24 +27,35 @@ namespace System.Xml.Private
 using System;
 using System.Xml;
 using System.Xml.XPath;
+using System.Collections;
 
 
 internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 {
 	private XmlNode node;
+	private XmlAttribute nsAttr = null;
 	private XmlDocument document;
 
+	/* xml:xmlns="http://www.w3.org/XML/1998/namespace" */
+	private XmlAttribute xmlAttr = null;
+
+	private ArrayList nsNames = null;
+
 	public XmlDocumentNavigator(XmlNode node) : base()
 	{
 		this.node = node;
 		this.document = (node is XmlDocument) ? 
 							(XmlDocument)node : node.OwnerDocument;
+		this.xmlAttr = document.CreateAttribute("xmlns", "xml", 
+												XmlDocument.xmlns);
+		this.xmlAttr.Value = XmlDocument.xmlnsXml;
+
+		this.nsNames = null;
 	}
 
 	public XmlDocumentNavigator(XmlDocumentNavigator copy)
 	{
-		this.node = copy.node;
-		this.document = copy.document;
+		this.MoveTo(copy);
 	}
 
 	public override XPathNavigator Clone()
@@ -68,11 +79,11 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 	{
 		return node.GetNamespaceOfPrefix(name);
 	}
-	
+
 	public override bool IsSamePosition(XPathNavigator other)
 	{
 		XmlDocumentNavigator nav = (other as XmlDocumentNavigator);
-		return ((nav != null) && (nav.node == node));
+		return ((nav != null) && (nav.node == node) && (nav.nsAttr == nsAttr));
 	}
 	
 	public override bool MoveTo(XPathNavigator other)
@@ -81,7 +92,17 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 		if(nav != null)
 		{
 			node = nav.node;
+			nsAttr = nav.nsAttr;
 			document = nav.document;
+			xmlAttr = nav.xmlAttr;
+			if(nav.nsNames == null || nav.nsNames.IsReadOnly)
+			{
+				nsNames = nav.nsNames;
+			}
+			else
+			{
+				nsNames = ArrayList.ReadOnly(nav.nsNames);
+			}
 			return true;
 		}
 		return false;
@@ -98,6 +119,7 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 					attr.NamespaceURI == namespaceURI)
 				{
 					node = attr;
+					NamespaceAttribute = null;
 					return true;
 				}
 			}
@@ -120,10 +142,14 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 	{
 		if(NodeType == XPathNodeType.Element && node.Attributes != null)
 		{
-			if(node.Attributes.Count != 0)
+			foreach(XmlAttribute attr in node.Attributes)
 			{
-				node = node.Attributes[0];
-				return true;
+				if(attr.NamespaceURI != XmlDocument.xmlns)
+				{
+					node = attr;
+					NamespaceAttribute = null;
+					return true;
+				}
 			}
 		}
 		return false;
@@ -131,6 +157,11 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 
 	public override bool MoveToFirstChild()
 	{
+		if(!HasChildren)
+		{
+			return false;
+		}
+
 		XmlNode next = node.FirstChild;
 		// TODO: implement normalization
 		while(next!= null && 
@@ -149,8 +180,45 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 		return false;
 	}
 
-	public override bool MoveToFirstNamespace(XPathNamespaceScope namespaceScope)
+	public override bool MoveToFirstNamespace(
+								XPathNamespaceScope namespaceScope)
 	{
+		if(NodeType != XPathNodeType.Element)
+		{
+			return false;
+		}
+
+		XmlElement element = (XmlElement)node;
+		while(element != null)
+		{
+			if(element.Attributes != null)
+			{
+				foreach(XmlAttribute attr in element.Attributes)
+				{
+					if(attr.NamespaceURI == XmlDocument.xmlns 
+						&& !CheckForDuplicateNS(attr.Name, attr.Value))
+					{
+						NamespaceAttribute = attr;
+						return true;
+					}
+				}
+			}
+			if(namespaceScope == XPathNamespaceScope.Local)
+			{
+				return false;
+			}
+
+			element = element.ParentNode as XmlElement;
+		}
+
+		if(namespaceScope == XPathNamespaceScope.All)
+		{
+			if(!CheckForDuplicateNS(xmlAttr.Name, xmlAttr.Value))
+			{
+				NamespaceAttribute = xmlAttr;
+				return true;
+			}
+		}
 		return false;
 	}
 
@@ -161,11 +229,47 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 
 	public override bool MoveToNamespace(String name)
 	{
+		if(name == "xml")
+		{
+			/* seems to be that xml namespaces are valid
+			   wherever you are ? */
+			NamespaceAttribute = xmlAttr;
+			return true;
+		}
+		
+		if(NodeType != XPathNodeType.Element)
+		{
+			return false;
+		}
+
+		XmlElement element = (XmlElement)node;
+		while(element != null)
+		{
+			if(element.Attributes != null)
+			{
+				foreach(XmlAttribute attr in element.Attributes)
+				{
+					if(attr.NamespaceURI == XmlDocument.xmlns
+						&& !CheckForDuplicateNS(attr.Name,attr.Value))
+					{
+						NamespaceAttribute = attr;
+						return true;
+					}
+				}
+			}
+			element = element.ParentNode as XmlElement;
+		}
+
 		return false;
 	}
 
 	public override bool MoveToNext()
 	{
+		if(nsAttr != null)
+		{
+			return false;
+		}
+
 		XmlNode next = node.NextSibling;
 		// TODO: implement normalization
 		while(next!= null && 
@@ -217,32 +321,120 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 			if(i != list.Count)
 			{
 				node = list[i];
+				NamespaceAttribute = null;
 				return true;
 			}
 		}
 		return false;
 	}
 
+	private XmlAttribute GetNextNamespace(XmlElement owner, 
+											XmlAttribute current)
+	{
+		for(int i = 0; i < owner.Attributes.Count; i++)
+		{
+			XmlAttribute attr = owner.Attributes[i];
+			if(((Object)attr) == ((Object)current) || current == null)
+			{
+				for(int j = i+1; j < owner.Attributes.Count; j++)
+				{
+					attr = owner.Attributes[j];
+					if(attr.NamespaceURI == XmlDocument.xmlns 
+						&& !CheckForDuplicateNS(attr.Name, attr.Value))
+					{
+						return attr;	
+					}
+				}
+			}
+		}
+
+		return null;
+	}
+
 	public override bool MoveToNextNamespace(XPathNamespaceScope namespaceScope)
 	{
+		if(nsAttr == null)
+		{
+			return false;
+		}
+		
+		XmlElement owner = nsAttr.OwnerElement;
+		
+		if(owner == null || ((Object)nsAttr) == ((Object)this.xmlAttr)) 
+		{
+			/* technically, I don't think we need the extra condition
+			   because xmlAttr won't be attached to an element, but
+			   it's there because it makes it clear :) */
+			return false;
+		}
+
+		XmlAttribute nextNs = GetNextNamespace(owner, nsAttr);
+
+		if(nextNs != null)
+		{
+			NamespaceAttribute = nextNs;
+			return true;
+		}
+
+		if(namespaceScope == XPathNamespaceScope.Local)
+		{
+			return false;
+		}
+		
+		for(XmlNode node = owner.ParentNode;
+				node != null; node = node.ParentNode)
+		{
+			owner = (node as XmlElement);
+			if(owner == null)
+			{
+				continue;
+			}
+			nextNs = GetNextNamespace(owner, null);
+			if(nextNs != null)
+			{
+				NamespaceAttribute = nextNs;
+				return true;
+			}
+		}
+		
+		if(namespaceScope == XPathNamespaceScope.All)
+		{
+			if(!CheckForDuplicateNS(xmlAttr.Name, xmlAttr.Value))
+			{
+				NamespaceAttribute = xmlAttr;
+				return true;
+			}
+		}
 		return false;
 	}
 	
 
 	public override bool MoveToParent()
 	{
+		if(nsAttr != null)
+		{
+			/* the scary part is the MoveToNextNamespace
+			   function where you just traverse up. So
+			   there is no guarantee that parent node of
+			   nsAttr is the next node you want */
+			NamespaceAttribute = null;
+			return true;
+		}
+
 		if(node.NodeType == XmlNodeType.Attribute)
 		{
 			XmlElement owner = ((XmlAttribute)node).OwnerElement;
 			if(owner != null)
 			{
 				node = owner;
+				NamespaceAttribute = null;
 				return true;
 			}
 		}
 		else if (node.ParentNode != null)
 		{
 			node = node.ParentNode;
+			NamespaceAttribute = null;
 			return true;
 		}
 		return false;
@@ -250,6 +442,11 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 
 	public override bool MoveToPrevious()
 	{
+		if(nsAttr != null)
+		{
+			return true;
+		}
+
 		if(node.PreviousSibling != null)
 		{
 			node = node.PreviousSibling;
@@ -265,6 +462,7 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 		{
 			node = document;
 		}
+		NamespaceAttribute = null;
 		return;
 	}
 	
@@ -280,7 +478,7 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 	{
 		get
 		{
-			if(node.Attributes != null)
+			if(nsAttr == null && node.Attributes != null)
 			{
 				return (node.Attributes.Count != 0);
 			}
@@ -292,7 +490,7 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 	{
 		get
 		{
-			return (node.FirstChild != null);	
+			return (nsAttr == null && node.FirstChild != null);	
 		}
 	}
 
@@ -300,7 +498,7 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 	{
 		get
 		{
-			if(node.NodeType == XmlNodeType.Element)
+			if(nsAttr == null && node.NodeType == XmlNodeType.Element)
 			{
 				return ((XmlElement)node).IsEmpty;
 			}
@@ -316,11 +514,14 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 			
 			if(nodeType == XPathNodeType.Element ||
 				nodeType == XPathNodeType.Attribute ||
-				nodeType == XPathNodeType.ProcessingInstruction ||
-				nodeType == XPathNodeType.Namespace)
+				nodeType == XPathNodeType.ProcessingInstruction)
 			{
 				return node.LocalName;
 			}
+			else if(nodeType == XPathNodeType.Namespace)
+			{
+				return nsAttr.LocalName;
+			}
 			return String.Empty;
 		}
 	}
@@ -333,11 +534,14 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 			
 			if(nodeType == XPathNodeType.Element ||
 				nodeType == XPathNodeType.Attribute ||
-				nodeType == XPathNodeType.ProcessingInstruction ||
-				nodeType == XPathNodeType.Namespace)
+				nodeType == XPathNodeType.ProcessingInstruction)
 			{
 				return node.Name;
 			}
+			else if(nodeType == XPathNodeType.Namespace)
+			{
+				return LocalName;
+			}
 			return String.Empty;
 		}
 	}
@@ -354,6 +558,10 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 	{
 		get
 		{
+			if(nsAttr != null) 
+			{
+				return String.Empty;
+			}
 			return node.NamespaceURI;
 		}
 	}
@@ -362,6 +570,11 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 	{
 		get
 		{
+			if(nsAttr != null)
+			{
+				return XPathNodeType.Namespace;
+			}
+
 			switch(node.NodeType)
 			{
 				case XmlNodeType.Element:
@@ -449,8 +662,7 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 				break;
 				case XPathNodeType.Namespace:
 				{
-					// TODO ?
-					return String.Empty;
+					return nsAttr.Value; 
 				}
 				break;
 			}
@@ -475,16 +687,82 @@ internal class XmlDocumentNavigator : XPathNavigator, IHasXmlNode
 	{
 		get
 		{
+			if(this.nsAttr != null) 
+			{
+				return this.nsAttr;
+			}
 			return this.node;
 		}
 	}
 
+	internal XmlAttribute NamespaceAttribute 
+	{
+		get
+		{
+			return this.nsAttr;
+		}
+		set
+		{
+			this.nsAttr = value;
+
+			if(value != null)
+			{
+				if(this.nsNames == null)
+				{
+					this.nsNames = new ArrayList();
+				}
+				else if(this.nsNames.IsReadOnly)
+				{
+					this.nsNames = new ArrayList(this.nsNames);
+				}
+
+				this.nsNames.Add(value.Value);
+			}
+			else
+			{
+				this.nsNames = null;
+			}
+		}
+	}
+
+	// return true if the namespace has been seen before 
+	private bool CheckForDuplicateNS(String name, String ns)
+	{
+		// XmlNameTable to the rescue, we can compare names as objects
+		if(this.nsNames != null && this.nsNames.Contains((Object)name))
+		{
+			// duplicate 
+			return true;
+		}
+
+		/* tricky part: setting xmlns='' in your XML causes 
+		   the default namespace to be *seen* but forgotten.
+		*/
+		
+		if(ns == String.Empty)
+		{
+			if(this.nsNames == null)
+			{
+				this.nsNames = new ArrayList();
+			}
+			else if(this.nsNames.IsReadOnly)
+			{
+				this.nsNames = new ArrayList(this.nsNames);
+			}
+			this.nsNames.Add(NameTable.Get("xmlns"));
+
+			return true;
+		}
+
+		return false;
+	}
+
 	XmlNode IHasXmlNode.GetNode()
 	{
-		return this.node;
+		return CurrentNode;
 	}
 }
 
-#endif /* CONFIG_XPATH */
+#endif // CONFIG_XPATH
 
 }
