@@ -44,7 +44,7 @@ using System.Configuration;
 
 namespace System.Xml.Serialization
 {
-
+#if CONFIG_SERIALIZATION
 	public class XmlSerializer
 	{
 		internal const string WsdlNamespace = "http://schemas.xmlsoap.org/wsdl/";
@@ -438,12 +438,28 @@ namespace System.Xml.Serialization
 				}
 			}
 			
+			/*
+			Brubbel:
+			we have problems using the generator.
+			if we do not generate it no problems where there any more.
+			The problems are that the compiled generator does not work or 
+			the compiler returns an error.
+			Since we do not generate any more, we never had problems with 
+			serialization and deserialization of objects.
+			 
+			Since we have libjit the generator is no longer needed.
+			
+			Please be careful if U want to enable this.
+			*/
+			
+#if USE_GENERATOR
 			bool generate = false;
 			lock (serializerData)
 			{
 				generate = (++serializerData.UsageCount == generationThreshold);
 			}
 			
+			
 			if (generate)
 			{
 				if (serializerData.Batch != null)
@@ -456,6 +472,7 @@ namespace System.Xml.Serialization
 					GenerateSerializers (batch);
 				}
 			}
+#endif
 		}
 		
 		void GenerateSerializers (GenerationBatch batch)
@@ -570,4 +587,5 @@ namespace System.Xml.Serialization
 		
 #endregion // Methods
 	}
+#endif // CONFIG_SERIALIZATION
 }
