@@ -22,6 +22,8 @@
 namespace System.Xml.XPath
 {
 
+#if CONFIG_XPATH
+
 #if ECMA_COMPAT
 internal
 #else
@@ -34,4 +36,6 @@ interface IXPathNavigable
 
 }; // interface IXPathNavigable
 
+#endif // CONFIG_XPATH
+
 }; // namespace System.Xml.XPath
