@@ -39,4 +39,4 @@ enum XPathNamespaceScope
 }
 }//namespace
 
-#endif /* CONFIG_XPATH */
+#endif // CONFIG_XPATH
