@@ -38,4 +38,4 @@ enum XmlSortOrder
 }
 }//namespace
 
-#endif /* CONFIG_XPATH */
+#endif // CONFIG_XPATH
