@@ -43,6 +43,8 @@ class XmlDocument : XmlNode
 	internal XmlDocumentFragment placeholder;
 	internal XmlImplementation implementation;
 	internal static readonly String xmlns = "http://www.w3.org/2000/xmlns/";
+	/* is it me or did they a miss a trailing '/' here ? */
+	internal static readonly String xmlnsXml = "http://www.w3.org/XML/1998/namespace";
 
 	// Constructors.
 	public XmlDocument() : base(null)
@@ -685,7 +687,8 @@ class XmlDocument : XmlNode
 				{
 					throw new XmlException(S._("XmlException_NoXml"));
 				}
-
+				
+				RemoveAll();
 				XmlTextReader reader = new XmlTextReader(xml, XmlNodeType.Document, null);
 				BuildStructure(reader);
 			}
@@ -844,6 +847,8 @@ class XmlDocument : XmlNode
 							return elem;
 						}
 
+						elem.IsEmpty = isEmptyElement;
+
 						// read and append the children
 						ReadChildren(r, elem);
 
@@ -931,7 +936,7 @@ class XmlDocument : XmlNode
 							{
 								if(r.NodeType != XmlNodeType.Whitespace)
 								{
-									return ReadNodeInternal(r);
+									return null;
 								}
 							}
 							return null;
