@@ -26,14 +26,20 @@ using System;
 using System.Collections;
 using System.Text;
 using System.Xml.Private;
+
+#if CONFIG_XPATH
 using System.Xml.XPath;
+#endif
 
 #if ECMA_COMPAT
 internal
 #else
 public
 #endif
-abstract class XmlNode : ICloneable, IEnumerable, IXPathNavigable
+abstract class XmlNode : ICloneable, IEnumerable
+#if CONFIG_XPATH
+, IXPathNavigable
+#endif
 {
 	// Internal state.
 	internal XmlNode  parent;
@@ -370,6 +376,10 @@ abstract class XmlNode : ICloneable, IEnumerable, IXPathNavigable
 				{
 					return String.Empty;
 				}
+				set
+				{
+					throw new NotSupportedException("Prefix");
+				}
 			}
 
 	// Get the previous sibling of this node.
@@ -498,13 +508,13 @@ abstract class XmlNode : ICloneable, IEnumerable, IXPathNavigable
 	// Clone this node in either shallow or deep mode.
 	public abstract XmlNode CloneNode(bool deep);
 
+#if CONFIG_XPATH
 	// Implement the IXPathNavigator interface.
-	[TODO]
 	public virtual XPathNavigator CreateNavigator()
 			{
-				// TODO
-				return null;
+				return new XmlDocumentNavigator(this);
 			}
+#endif // CONFIG_XPATH
 
 	// Implement the IEnumerable interface,
 	public IEnumerator GetEnumerator()
@@ -800,8 +810,15 @@ abstract class XmlNode : ICloneable, IEnumerable, IXPathNavigable
 
 				// Perform the insert.
 				newChild.parent = this;
-				refChild = NodeList.GetPreviousSibling(refChild);
-				NodeList.GetList(this).InsertAfter(newChild, refChild);
+				if( refChild == this.FirstChild ) {
+					NodeList.GetList(this).first = newChild;
+					NodeList.GetList(newChild).nextSibling = refChild;
+					NodeList.GetList(refChild).prevSibling = newChild;
+				}
+				else {
+					refChild = NodeList.GetPreviousSibling(refChild);
+					NodeList.GetList(this).InsertAfter(newChild, refChild);
+				}
 
 				// Notify the document after the insert.
 				EmitAfter(args);
@@ -1002,38 +1019,47 @@ abstract class XmlNode : ICloneable, IEnumerable, IXPathNavigable
 				return oldChild;
 			}
 
+#if CONFIG_XPATH
+
 	// Select a list of nodes matching a particular XPath expression.
-	[TODO]
 	public XmlNodeList SelectNodes(String xpath)
 			{
-				// TODO
-				return null;
+				return SelectNodes(xpath, null);
 			}
 
 	// Select a list of nodes matching a particular XPath expression.
-	[TODO]
 	public XmlNodeList SelectNodes(String xpath, XmlNamespaceManager nsmgr)
 			{
-				// TODO
-				return null;
+				XPathNavigator nav = CreateNavigator();
+				XPathExpression expr = nav.Compile(xpath);
+				if(nsmgr != null)
+				{
+					expr.SetContext(nsmgr);
+				}
+				return new SelectNodeList(nav.Select(expr));
 			}
 
 	// Select a single node matching a particular XPath expression.
-	[TODO]
 	public XmlNode SelectSingleNode(String xpath)
 			{
-				// TODO
-				return null;
+				return SelectSingleNode(xpath, null);
 			}
 
 	// Select a single node matching a particular XPath expression.
-	[TODO]
 	public XmlNode SelectSingleNode(String xpath, XmlNamespaceManager nsmgr)
 			{
-				// TODO
-				return null;
+				XPathNavigator nav = CreateNavigator();
+				XPathExpression expr = nav.Compile(xpath);
+				if(nsmgr != null)
+				{
+					expr.SetContext(nsmgr);
+				}
+				SelectNodeList list = new SelectNodeList(nav.Select(expr));
+				return (list.Count == 0 ? null : list[0]);
 			}
 
+#endif // CONFIG_XPATH
+
 	// Test if this implementation supports a particular DOM feature.
 	public virtual bool Supports(String feature, String version)
 			{
