@@ -30,6 +30,7 @@ public class XmlTextWriter : XmlWriter
 {
 	// Internal state.
 	private bool                namespaces;
+	private bool                needPrefixDecl;
 	private int                 indentation;
 	private int                 indentLevel;
 	private char                indentChar;
@@ -41,8 +42,8 @@ public class XmlTextWriter : XmlWriter
 	private SpecialWriter       specialWriter;
 	private String              nsPrefix;
 	private String              xmlLang;
-	private WriteState          writeState;
-	private XmlNameTable        nameTable;
+	protected WriteState        writeState; // need protected for XmlFragmentTextWriter
+	protected XmlNameTable      nameTable;  // need protected for XmlFragmentTextWriter
 	private XmlNamespaceManager namespaceManager;
 	private XmlSpace            xmlSpace;
 	internal bool               autoShiftToContent;
@@ -86,6 +87,7 @@ public class XmlTextWriter : XmlWriter
 	private void Initialize()
 			{
 				namespaces = true;
+				needPrefixDecl = false;
 				indentation = 2;
 				indentLevel = 0;
 				indentChar = ' ';
@@ -272,6 +274,14 @@ public class XmlTextWriter : XmlWriter
 						// Add the mapping to the namespace manager.
 						namespaceManager.AddNamespace(nsPrefix, value);
 
+						// Check for match to required prefix declaration.
+						if(needPrefixDecl && nsPrefix == scope.prefix &&
+						   value == scope.xmlns)
+						{
+							// Flag that required prefix declaration is present.
+							needPrefixDecl = false;
+						}
+
 						// Reset the namespace prefix.
 						nsPrefix = null;
 					}
@@ -289,24 +299,45 @@ public class XmlTextWriter : XmlWriter
 			}
 
 	// Handle the start of a special attribute.
-	private void SpecialAttributeStart(bool space)
+	private void SpecialAttributeStart
+				(String prefix, String localName, Special type)
 			{
 				// Set the type of the special attribute.
-				special = (space ? Special.Space : Special.Lang);
+				special = type;
+
+				// Write the attribute prefix, if needed.
+				if(((Object)prefix) == null)
+				{
+					// Write the name.
+					writer.Write("xmlns");
+
+					// Set the local name.
+					localName = String.Empty;
+				}
+				else
+				{
+					// Write the prefixed attribute name.
+					writer.Write(prefix);
+					writer.Write(':');
+					writer.Write(localName);
+				}
+
+				// Set the namespace prefix, if needed.
+				if(special == Special.Namespace)
+				{
+					// Set the namespace prefix.
+					nsPrefix = localName;
+				}
+
+				// Output the start of the attribute value.
+				writer.Write('=');
+				writer.Write(quoteChar);
 
 				// Set the writer.
 				writer = specialWriter;
-			}
-	private void SpecialAttributeStart(String prefix)
-			{
-				// Set the namespace prefix.
-				nsPrefix = prefix;
 
-				// Set the type of the special attribute.
-				special = Special.Namespace;
-
-				// Set the writer.
-				writer = specialWriter;
+				// We are now in the attribute state.
+				writeState = System.Xml.WriteState.Attribute;
 			}
 
 	// Synchronize the output with a particular document area.
@@ -323,7 +354,10 @@ public class XmlTextWriter : XmlWriter
 					{
 						if((flags & WriteStateFlag.ContentFlag) != 0)
 						{
-							writer.Write(">");
+							// Terminate the element.
+							TerminateElement(false);
+
+							// Switch to contents.
 							writeState = System.Xml.WriteState.Content;
 						}
 						else
@@ -342,9 +376,13 @@ public class XmlTextWriter : XmlWriter
 						}
 						else if((flags & WriteStateFlag.ContentFlag) != 0)
 						{
-							// Terminate the attribute and switch to contents.
-							writer.Write(quoteChar);
-							writer.Write(">");
+							// Terminate the attribute.
+							TerminateAttribute();
+
+							// Terminate the element.
+							TerminateElement(false);
+
+							// Switch to contents.
 							writeState = System.Xml.WriteState.Content;
 						}
 						else
@@ -424,14 +462,51 @@ public class XmlTextWriter : XmlWriter
 				}
 			}
 
+	// Terminate an attribute.
+	private void TerminateAttribute()
+			{
+				// Handle the end of a special attribute, if needed.
+				if(special != Special.None) { SpecialAttributeEnd(); }
+
+				// Terminate the attribute.
+				writer.Write(quoteChar);
+			}
+
+	// Terminate a start element.
+	private void TerminateElement(bool empty)
+			{
+				// Write the prefix declaration, if needed.
+				if(needPrefixDecl)
+				{
+					writer.Write("xmlns:");
+					writer.Write(scope.prefix);
+					writer.Write('=');
+					writer.Write(quoteChar);
+					WriteQuotedString(scope.xmlns);
+					writer.Write(quoteChar);
+					needPrefixDecl = false;
+				}
+
+				// Terminate the element.
+				if(empty)
+				{
+					// Terminate the empty element.
+					writer.Write(" />");
+				}
+				else
+				{
+					// Terminate the start element.
+					writer.Write('>');
+				}
+			}
 
 	private void WriteRawData(String value)
-	{
-		writer.Write(value);
-	}
+			{
+				writer.Write(value);
+			}
 
 	// Write an xml declaration.
-	private void WriteXmlDeclaration(String text)
+	virtual protected void WriteXmlDeclaration(String text)
 			{
 				// make sure we're in the start state
 				if(writeState != System.Xml.WriteState.Start)
@@ -887,11 +962,8 @@ public class XmlTextWriter : XmlWriter
 						(S._("Xml_InvalidWriteState"));
 				}
 
-				// Handle the end of a special attribute, if needed.
-				if(special != Special.None) { SpecialAttributeEnd(); }
-
 				// Terminate the attribute and return to the element state.
-				writer.Write(quoteChar);
+				TerminateAttribute();
 				writeState = System.Xml.WriteState.Element;
 			}
 
@@ -938,8 +1010,8 @@ public class XmlTextWriter : XmlWriter
 				// We must be in the Element or Content state.
 				if(writeState == System.Xml.WriteState.Element)
 				{
-					// Terminate the element with " />".
-					writer.Write(" />");
+					// Terminate the empty element.
+					TerminateElement(true);
 				}
 				else if(writeState == System.Xml.WriteState.Content)
 				{
@@ -988,7 +1060,7 @@ public class XmlTextWriter : XmlWriter
 				if(writeState == System.Xml.WriteState.Element)
 				{
 					// Terminate the start element.
-					writer.Write('>');
+					TerminateElement(false);
 				}
 				if(writeState == System.Xml.WriteState.Element ||
 				   writeState == System.Xml.WriteState.Content)
@@ -1254,11 +1326,8 @@ public class XmlTextWriter : XmlWriter
 				// Check the state and output delimiters.
 				if(writeState == System.Xml.WriteState.Attribute)
 				{
-					// Handle the end of a special attribute, if needed.
-					if(special != Special.None) { SpecialAttributeEnd(); }
-
-					// Write the closing quote character for the attribute.
-					writer.Write(quoteChar);
+					// Terminate the attribute.
+					TerminateAttribute();
 
 					// Write a space before the start of the attribute.
 					writer.Write(' ');
@@ -1284,7 +1353,7 @@ public class XmlTextWriter : XmlWriter
 						// Ensure the namespace is correct.
 						if(nsLen != 0 &&
 						   (nsLen != 29 ||
-						    ns != "http://www.w3.org/2000/xmlns/"))
+						    ns != XmlDocument.xmlns))
 						{
 							throw new ArgumentException(/* TODO */);
 						}
@@ -1300,14 +1369,18 @@ public class XmlTextWriter : XmlWriter
 						}
 
 						// Handle the start of the special attribute.
-						SpecialAttributeStart(localName);
+						SpecialAttributeStart
+							(prefix, localName, Special.Namespace);
+
+						// We're done here.
+						return;
 					}
 					else if(prefixLen == 3 && prefix == "xml")
 					{
 						// Ensure the namespace is correct.
 						if(nsLen != 0 &&
 						   (nsLen != 36 ||
-						    ns != "http://www.w3.org/XML/1998/namespace"))
+						    ns != XmlDocument.xmlnsXml))
 						{
 							throw new ArgumentException(/* TODO */);
 						}
@@ -1320,12 +1393,20 @@ public class XmlTextWriter : XmlWriter
 						if(nameLen == 4 && localName == "lang")
 						{
 							// Handle the start of the special attribute.
-							SpecialAttributeStart(false);
+							SpecialAttributeStart
+								(prefix, localName, Special.Lang);
+
+							// We're done here.
+							return;
 						}
 						else if(nameLen == 5 && localName == "space")
 						{
 							// Handle the start of the special attribute.
-							SpecialAttributeStart(true);
+							SpecialAttributeStart
+								(prefix, localName, Special.Space);
+
+							// We're done here.
+							return;
 						}
 					}
 					else if(nsLen != 0)
@@ -1371,21 +1452,24 @@ public class XmlTextWriter : XmlWriter
 					   (localName.Length == 5 && localName == "xmlns"))
 					{
 						// Ensure the namespace is correct.
-						if(nsLen != 29 || ns != "http://www.w3.org/2000/xmlns/")
+						if(nsLen != 29 || ns != XmlDocument.xmlns)
 						{
 							throw new ArgumentException(/* TODO */);
 						}
 
 						// Handle the start of the special attribute.
-						SpecialAttributeStart(String.Empty);
+						SpecialAttributeStart(null, null, Special.Namespace);
+
+						// We're done here.
+						return;
 					}
 					else if(nsLen == 29 &&
-					        ns == "http://www.w3.org/2000/xmlns/")
+					        ns == XmlDocument.xmlns)
 					{
 						throw new ArgumentException(/* TODO */);
 					}
 					else if(nsLen == 36 &&
-					        ns == "http://www.w3.org/XML/1998/namespace")
+					        ns == XmlDocument.xmlnsXml)
 					{
 						throw new ArgumentException(/* TODO */);
 					}
@@ -1504,16 +1588,14 @@ public class XmlTextWriter : XmlWriter
 				{
 					// We need to associate a prefix with a namespace.
 					String currMapping = LookupPrefix(ns);
+					
 					if(currMapping == prefix)
 					{
-						// The prefix is already mapped to this URI.
-						if(prefix != currPrefix)
-						{
-							writer.Write(prefix);
-							writer.Write(':');
-							scopeShown = true;
-						}
+						// always write prefix
+						writer.Write(prefix);
+						writer.Write(':');
 						writer.Write(localName);
+						scopeShown = true;
 					}
 					else
 					{
@@ -1522,12 +1604,7 @@ public class XmlTextWriter : XmlWriter
 						writer.Write(':');
 						writer.Write(localName);
 						writer.Write(' ');
-						writer.Write("xmlns:");
-						writer.Write(prefix);
-						writer.Write('=');
-						writer.Write(quoteChar);
-						WriteQuotedString(ns);
-						writer.Write(quoteChar);
+						needPrefixDecl = true;
 						scopeShown = true;
 					}
 				}
@@ -1548,33 +1625,8 @@ public class XmlTextWriter : XmlWriter
 					prefix = LookupPrefix(ns);
 					if(((Object)prefix) == null || prefix.Length == 0)
 					{
-						if(scope == null)
-						{
-							writer.Write(localName);
-							writer.Write(' ');
-							writer.Write("xmlns");
-							writer.Write('=');
-							writer.Write(quoteChar);
-							WriteQuotedString(ns);
-							writer.Write(quoteChar);
-							scopeShown = true;
-						}
-						else if(scope.scopeShown == false || scope.xmlns != ns)
-						{
-							writer.Write(localName);
-							writer.Write(' ');	
-							writer.Write("xmlns");
-							writer.Write('=');
-							writer.Write(quoteChar);
-							WriteQuotedString(ns);
-							writer.Write(quoteChar);
-							scopeShown = true;
-						}
-						else if(scope.xmlns == ns)
-						{
-							writer.Write(localName);
-							scopeShown = true;
-						}
+						writer.Write(localName);
+						scopeShown = true;
 					}
 					else if(prefix != currPrefix)
 					{
