@@ -23,6 +23,8 @@
 
 #if !ECMA_COMPAT
 
+#if CONFIG_XPATH && CONFIG_XSL
+
 using System;
 using System.Xml.XPath;
 
@@ -44,4 +46,5 @@ namespace System.Xml.Xsl
 	}
 }//namespace
 
-#endif
+#endif // CONFIG_XPATH && CONFIG_XSL
+#endif // !ECMA_COMPAT
