@@ -38,10 +38,8 @@ namespace System.Xml.Xsl
 			throw new NotImplementedException(".ctor");
 		}
 
-		[TODO]
 		public XsltContext(NameTable table) : base(table)
 		{
-			throw new NotImplementedException(".ctor");
 		}
 
 		public abstract int CompareDocument(String baseUri, 
@@ -59,5 +57,5 @@ namespace System.Xml.Xsl
 
 	}
 }//namespace
-#endif /* CONFIG_XPATH && CONFIG_XSL */
-#endif /* !ECMA_COMPAT */
+#endif // CONFIG_XPATH && CONFIG_XSL
+#endif // !ECMA_COMPAT
