@@ -1,7 +1,7 @@
 /*
  * SerialPort.cs - Implementation of the "System.IO.Ports.SerialPort" class.
  *
- * Copyright (C) 2003  Southern Storm Software, Pty Ltd.
+ * Copyright (C) 2003-2004  Southern Storm Software, Pty Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@ public class SerialPort
 		 3000000, 3500000, 4000000};
 
 	// Timeout value that indicates an infinite timeout.
-	public static readonly int InfiniteTimeout = -1;
+	public static readonly int InfiniteTimeout = 0;
 
 	// Constructors.
 	public SerialPort()
@@ -78,7 +78,7 @@ public class SerialPort
 				parameters.readTimeout = InfiniteTimeout;
 				parameters.writeTimeout = InfiniteTimeout;
 				baseStream = null;
-				newLine = "\r\n";
+				newLine = "\n";  // MS defaults to \n
 				newLineBuffer = null;
 			}
 	[TODO]
@@ -529,6 +529,17 @@ public class SerialPort
 						posn = 6;
 						type = PortMethods.SERIAL_INFRARED;
 					}
+					else if(value.Length > 6 &&
+					        (value[0] == 'r' || value[0] == 'R') &&
+					        (value[1] == 'f' || value[1] == 'F') &&
+					        (value[2] == 'c' || value[2] == 'C') &&
+					        (value[3] == 'o' || value[3] == 'O') &&
+					        (value[4] == 'm' || value[4] == 'M') &&
+					        (value[5] == 'm' || value[5] == 'M'))
+					{
+						posn = 6;
+						type = PortMethods.SERIAL_RFCOMM;
+					}
 					else if(value.Length > 3 &&
 					        (value[0] == 'u' || value[0] == 'U') &&
 					        (value[1] == 's' || value[1] == 'S') &&
@@ -861,6 +872,15 @@ public class SerialPort
 	public String ReadExisting()
 			{
 				// TODO
+				int toRead = BytesToRead;
+				if(toRead > 0)
+				{
+					byte[] buffer = new byte[toRead];
+					if(Read(buffer, 0, toRead) > 0)
+					{
+						return encoding.GetString(buffer);
+					}
+				}
 				return null;
 			}
 
@@ -868,6 +888,77 @@ public class SerialPort
 	[TODO]
 	public String ReadLine()
 			{
+				lock(this)
+				{
+					if(baseStream != null)
+					{
+						int[] backbuffer = new int[this.newLine.Length];
+						int bytesinbuffer = 0;
+						int outbufferpos = 0;
+						bool pendingeol = false;
+						bool eolfailed = false;
+						StringBuilder inLine = new StringBuilder();
+						while(true)
+						{
+							int retByte;
+							
+							// read the next byte
+							if(bytesinbuffer > 0 && eolfailed == true)
+							{
+								retByte = backbuffer[outbufferpos++];
+								if(--bytesinbuffer == 0)
+								{
+									// backbuffer is flushed. reset for next multibyte eol search
+									eolfailed = false;
+									outbufferpos = 0;
+								}
+							}
+							else
+							{
+								retByte = baseStream.ReadByte();
+							}
+							
+							// is this an eol byte?
+							if(retByte == this.newLine[bytesinbuffer])
+							{
+								backbuffer[bytesinbuffer++] = retByte;
+								if(this.newLine.Length == bytesinbuffer)
+								{
+									bytesinbuffer = 0;
+									return(inLine.ToString());
+								}
+								else
+								{
+									pendingeol = true;
+								}
+							}
+							else
+							{
+								if(bytesinbuffer > 0)
+								{
+									// eol search failed after finding at least one of a multi
+									// byte eol marker.  set eolfailed so that we flush the bytes
+									// that we stored during failed multi byte eol match
+									eolfailed = true;
+								}
+								
+							 	if(retByte == -1)
+								{
+									// FIXME need to store the inLine data so that we do not lose it
+									// if another ReadLine is issued after this timeout
+									return null;
+								}
+								
+								inLine.Append((char) retByte);						
+							}
+						}
+					}
+					else
+					{
+						throw new InvalidOperationException
+							(S._("Invalid_PortNotOpen"));
+					}
+				}
 				// TODO
 				return null;
 			}
@@ -998,7 +1089,7 @@ public class SerialPort
 		public PortStream(SerialPort port)
 				{
 					this.port = port;
-					this.byteBuffer = new byte [0];
+					this.byteBuffer = new byte [1];
 				}
 
 		// Close the stream.
