@@ -30,6 +30,12 @@ public enum SocketFlags
 	Peek				= 0x0002,
 	DontRoute			= 0x0004,
 	MaxIOVectorLength	= 0x0010,
+#if CONFIG_FRAMEWORK_2_0 && !CONFIG_COMPACT_FRAMEWORK
+	Truncated			= 0x0100,
+	ControlDataTruncated= 0x0200,
+	Broadcast			= 0x0400,
+	Multicast			= 0x0800,
+#endif // CONFIG_FRAMEWORK_2_0 && !CONFIG_COMPACT_FRAMEWORK
 	Partial				= 0x8000
 
 }; // enum SocketFlags
