@@ -248,6 +248,12 @@ public sealed class Application : IDisposable
 				cmdLineArgs = (String[])(newArgs.ToArray(typeof(String)));
 
 				// Connect to the display.
+				if(displayName == null)
+				{
+					// Xlib will figure it by itself, but classes using displayName can get broken is it's null
+					displayName = Environment.GetEnvironmentVariable("DISPLAY");
+				}
+
 				display = Xsharp.Display.Open(displayName, this, synchronous);
 
 				// Create the default font.
@@ -543,7 +549,7 @@ public sealed class Application : IDisposable
 					Xlib._XGetHostname(buf, 1024);
 					String host = Marshal.PtrToStringAnsi(buf);
 					Marshal.FreeHGlobal(buf);
-					if(host == null || host == String.Empty)
+					if(host == null || (host.Length == 0))
 					{
 						return null;
 					}
