@@ -20,7 +20,7 @@
 
 namespace Xsharp.Dcop
 {
-
+using System;
 // This class is not ECMA-compatible strictly speaking, but it is
 // needed to support thread-static variables in the ECMA engine.
 
@@ -31,6 +31,7 @@ namespace Xsharp.Dcop
 public class DcopCallAttribute : Attribute
 {
 	// FIXME: Tell me if this eats up memory, 'cause it's not really used for now
+	[NonSerializedAttribute]
 	DcopFunction function;
 	// Constructor.
 	public DcopCallAttribute(string fun)
