@@ -90,7 +90,7 @@ public class Font
 					this.family = family;
 				else
 					this.family = SansSerif;
-				if(pointSize < 0 || pointSize > 1000)
+				if(pointSize < 0 || pointSize > 10000)
 					this.pointSize = 120;
 				else
 					this.pointSize = pointSize;
@@ -731,6 +731,12 @@ public class Font
 	// that we don't get unsightly stretching.
 	internal static int NormalizePointSize(int pointSize)
 			{
+				// No need for normalization if using Xft
+				if (Xlib.XSharpUseXft () != 0)
+				{
+				    return pointSize;
+				}
+				
 				if(pointSize < 90)
 				{
 					return 80;
@@ -1397,10 +1403,20 @@ public class Font
 					try
 					{
 						IntPtr display = graphics.dpy.Lock();
-						Xlib.XSharpDrawStringXft
-								(display, graphics.drawableHandle, graphics.gc,
-								 fontSet, x, y, str, (int)style,
-								 IntPtr.Zero, graphics.Foreground.value);
+						if(graphics.clipRegion == null)
+						{
+							Xlib.XSharpDrawStringXft
+									(display, graphics.drawableHandle, graphics.gc,
+									 fontSet, x, y, str, (int)style,
+									 IntPtr.Zero, graphics.Foreground.value);
+						}
+						else
+						{
+							Xlib.XSharpDrawStringXft
+									(display, graphics.drawableHandle, graphics.gc,
+									 fontSet, x, y, str, (int)style,
+									 graphics.clipRegion.GetRegion(), graphics.Foreground.value);
+						}
 					}
 					finally
 					{
