@@ -69,6 +69,8 @@ public sealed class Graphics : IDisposable
 	private Bitmap stipple;
 	private byte[] dashPattern;
 	internal Region exposeRegion;
+	internal Region clipRegion;
+	private bool isDisposed;
 
 
 	/// <summary>
@@ -162,7 +164,7 @@ public sealed class Graphics : IDisposable
 						// Reset the cached GC back to the default settings.
 						// Xlib will take care of stripping the list down
 						// to just the changes that need to be applied.
-						gcValues.function = Xsharp.Function.GXcopy;
+						gcValues.function = Xsharp.GCFunction.GXcopy;
 						gcValues.plane_mask = ~((XPixel)0);
 						gcValues.line_width = 0;
 						gcValues.line_style = Xsharp.LineStyle.LineSolid;
@@ -224,6 +226,20 @@ public sealed class Graphics : IDisposable
 				{
 					((DoubleBuffer)drawable).ClearAtStart(this);
 				}
+
+				isDisposed = false;
+			}
+
+	/// <summary>
+	/// <para>Finalizer to clean up any managed resources by calling the
+	/// Dispose method.</para>
+	/// </summary>
+	~Graphics()
+			{
+				if(isDisposed == false)
+				{
+					this.Dispose();
+				}
 			}
 
 	/// <summary>
@@ -262,6 +278,11 @@ public sealed class Graphics : IDisposable
 					exposeRegion.Dispose();
 					exposeRegion = null;
 				}
+				if(clipRegion != null ) {
+					clipRegion.Dispose();
+					clipRegion = null;
+				}
+				isDisposed = true;
 			}
 
 	/// <summary>
@@ -404,13 +425,13 @@ public sealed class Graphics : IDisposable
 	/// </summary>
 	///
 	/// <value>
-	/// <para>The <see cref="T:Xsharp.Function"/> value for the mode.</para>
+	/// <para>The <see cref="T:Xsharp.GCFunction"/> value for the mode.</para>
 	/// </value>
 	///
 	/// <exception cref="T:Xsharp.XException">
 	/// <para>Raised if set to an invalid value.</para>
 	/// </exception>
-	public Xsharp.Function Function
+	public Xsharp.GCFunction Function
 			{
 				get
 				{
@@ -430,8 +451,8 @@ public sealed class Graphics : IDisposable
 				}
 				set
 				{
-					if(value < Xsharp.Function.GXclear ||
-					   value > Xsharp.Function.GXset)
+					if(value < Xsharp.GCFunction.GXclear ||
+					   value > Xsharp.GCFunction.GXset)
 					{
 						throw new XException
 							(String.Format(S._("X_Function"), (int)value));
@@ -1162,6 +1183,12 @@ public sealed class Graphics : IDisposable
 					IntPtr display = Lock();
 					Xlib.XSetClipOrigin(display, gc, 0, 0);
 					Xlib.XSetRegion(display, gc, r.GetRegion());
+					if(clipRegion != r)
+					{
+						if(clipRegion != null)
+							clipRegion.Dispose();
+						clipRegion = new Region(r);
+					}
 				}
 				finally
 				{
@@ -1211,6 +1238,12 @@ public sealed class Graphics : IDisposable
 					IntPtr display = Lock();
 					Xlib.XSetClipOrigin(display, gc, xorigin, yorigin);
 					Xlib.XSetRegion(display, gc, r.GetRegion());
+					if(clipRegion != r)
+					{
+						if(clipRegion != null)
+							clipRegion.Dispose();
+						clipRegion = new Region(r);
+					}
 				}
 				finally
 				{
@@ -2250,40 +2283,7 @@ public sealed class Graphics : IDisposable
 	/// </exception>
 	public void DrawImage(int x, int y, Image image)
 			{
-				// Validate the image parameter.
-				if(image == null)
-				{
-					throw new ArgumentNullException("image");
-				}
-
-				// Can we take a short-cut using the XImage?
-				if(image.ShouldUseXImage && !(drawable is Pixmap) &&
-				   !(drawable is DoubleBuffer))
-				{
-					// Use "PutXImage" to draw through the clip mask,
-					// to avoid having to create a Pixmap in the server.
-					int width, height;
-					SetFillSolid();
-					if (image.Mask != null)
-						SetClipMask(image.Mask, x, y);
-					Xlib.XSharpGetImageSize
-						(image.XImage, out width, out height);
-					PutXImage(image.XImage, 0, 0, x, y, width, height);
-				}
-				else
-				{
-					// Set the context to "tiling" and fill the region.
-					SetFillTiled(image.Pixmap, x, y);
-					if (image.Mask != null)
-						SetClipMask(image.Mask, x, y);
-					FillRectangle(x, y, image.Pixmap.Width,
-								  image.Pixmap.Height);
-				}
-
-				// Revert the context to a sane fill mode.
-				SetFillSolid();
-				if (image.Mask != null)
-					SetClipMask(null);
+				DrawImage(x,y, image, 0,0, image.Width, image.Height);
 			}
 
 	/// <summary>
@@ -2336,7 +2336,7 @@ public sealed class Graphics : IDisposable
 				// Validate the image parameter.
 				if(image == null)
 				{
-					throw new ArgumentNullException("image");
+					throw new ArgumentNullException("image", "Argument cannot be null");
 				}
 
 				// Bail out if the source co-ordinates are out of range.
@@ -2352,7 +2352,42 @@ public sealed class Graphics : IDisposable
 				{
 					throw new XException(S._("X_RectCoordRange"));
 				}
+				
+				// Intersect image with clip rect
+				Rectangle clipRect = clipRegion.ClipBox();
+				
+				int right = x + srcWidth;
+				int clipRight = clipRect.x + clipRect.width;
+				int bottom = y + srcHeight;
+				int clipBottom = clipRect.y + clipRect.height;
+				if(right < clipRect.x || x > clipRight || bottom < clipRect.y || y > clipBottom)
+					return;
 
+				if(right > clipRight)
+				{
+					srcWidth -= right - clipRight;
+				}
+				if(x < clipRect.x)
+				{
+					int i = clipRect.x - x;
+					srcX += i;
+					srcWidth -= i;
+					x += i;
+				}
+				
+
+				if(bottom > clipBottom)
+				{
+					srcHeight -= bottom - clipBottom;
+				}
+				if(y < clipRect.y)
+				{
+					int i = clipRect.y - y;
+					srcY += i;
+					srcHeight -= i;
+					y += i;
+				}
+	
 				// Can we take a short-cut using the XImage?
 				if(image.ShouldUseXImage && !(drawable is Pixmap) &&
 				   !(drawable is DoubleBuffer))
@@ -2360,7 +2395,7 @@ public sealed class Graphics : IDisposable
 					// Use "PutXImage" to draw through the clip mask,
 					// to avoid having to create a Pixmap in the server.
 					SetFillSolid();
-					SetClipMask(image.Mask, x - srcX, y - srcY);
+					if(image.Mask != null) SetClipMask(image.Mask, x - srcX, y - srcY);
 					PutXImage(image.XImage, srcX, srcY, x, y,
 							  srcWidth, srcHeight);
 				}
@@ -2368,13 +2403,14 @@ public sealed class Graphics : IDisposable
 				{
 					// Set the context to "tiling" and fill the region.
 					SetFillTiled(image.Pixmap, x - srcX, y - srcY);
-					SetClipMask(image.Mask, x - srcX, y - srcY);
+					if(image.Mask != null) SetClipMask(image.Mask, x - srcX, y - srcY);
 					FillRectangle(x, y, srcWidth, srcHeight);
 				}
 
 				// Revert the context to a sane fill mode.
 				SetFillSolid();
-				SetClipMask(null);
+				if(image.Mask != null) SetClipRegion(clipRegion);
+
 			}
 
 	/// <summary>
@@ -2533,7 +2569,7 @@ public sealed class Graphics : IDisposable
 				{
 					throw new ArgumentNullException("font");
 				}
-				if(str == null || str == String.Empty)
+				if(str == null || (str.Length == 0))
 				{
 					return;
 				}
@@ -2603,7 +2639,7 @@ public sealed class Graphics : IDisposable
 				{
 					throw new ArgumentNullException("font");
 				}
-				if(str == null || str == String.Empty)
+				if(str == null || (str.Length == 0))
 				{
 					width = 0;
 					ascent = 0;
