@@ -681,12 +681,20 @@ public class InputOutputWidget : InputOnlyWidget
 				if(region != null)
 				{
 					exposeRegion = null;
-					Graphics graphics = new Graphics(this);
-					graphics.exposeRegion = region;
-					graphics.SetClipRegion(region);
-					OnPaint(graphics);
-					graphics.Dispose();
-					region.Dispose();
+					// sometimes it could be that Expose is called but the handle is destroyed.
+					// so check here, if handle not null.
+					if( handle != XDrawable.Zero ) {
+						try {
+							Graphics graphics = new Graphics(this);
+							graphics.exposeRegion = region;
+							graphics.SetClipRegion(region);
+							OnPaint(graphics);
+							graphics.Dispose();
+							region.Dispose();
+						}
+						catch( XInvalidOperationException ) { // irgnore Widget disposed exception
+						}
+					}
 				}
 			}
 
