@@ -1,7 +1,7 @@
 /*
  * Stdio.cs - Implementation of the "Platform.Stdio" class.
  *
- * Copyright (C) 2001  Southern Storm Software, Pty Ltd.
+ * Copyright (C) 2001, 2004  Southern Storm Software, Pty Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -82,11 +82,28 @@ internal class Stdio
 	[MethodImpl(MethodImplOptions.InternalCall)]
 	extern public static void GetBufferSize(out int width, out int height);
 
+	// Set the buffer size.  May be ignored if the console doesn't
+	// support scroll-back buffering.
+	[MethodImpl(MethodImplOptions.InternalCall)]
+	extern public static void SetBufferSize(int width, int height);
+
 	// Get the window size and position information.
 	[MethodImpl(MethodImplOptions.InternalCall)]
 	extern public static void GetWindowSize
 			(out int left, out int top, out int width, out int height);
 
+	// Set the window size and position information.  This may be
+	// ignored if the system cannot alter the window size, or scroll-back
+	// buffers are not supported.
+	[MethodImpl(MethodImplOptions.InternalCall)]
+	extern public static void SetWindowSize
+			(int left, int top, int width, int height);
+
+	// Get the largest window size on this system.
+	[MethodImpl(MethodImplOptions.InternalCall)]
+	extern public static void GetLargestWindowSize
+			(out int width, out int height);
+
 	// Get the cursor position.
 	[MethodImpl(MethodImplOptions.InternalCall)]
 	extern public static void GetCursorPosition(out int x, out int y);
@@ -101,12 +118,16 @@ internal class Stdio
 
 	// Issue a beep on the console.
 	[MethodImpl(MethodImplOptions.InternalCall)]
-	extern public static void Beep();
+	extern public static void Beep(int frequency, int duration);
 
-	// Clear the console to the current foreground and backgroun colors.
+	// Clear the console to the current foreground and background colors.
 	[MethodImpl(MethodImplOptions.InternalCall)]
 	extern public static void Clear();
 
+	// Get the current foreground and background text attributes.
+	[MethodImpl(MethodImplOptions.InternalCall)]
+	extern public static int GetTextAttributes();
+
 	// Set the foreground and background text attributes.
 	[MethodImpl(MethodImplOptions.InternalCall)]
 	extern public static void SetTextAttributes(int attrs);
@@ -120,6 +141,39 @@ internal class Stdio
 	extern public static void ReadKey
 			(out char ch, out int key, out int modifiers);
 
+	// Move a buffer area.
+	[MethodImpl(MethodImplOptions.InternalCall)]
+	extern public static void MoveBufferArea
+			(int sourceLeft, int sourceTop,
+			 int sourceWidth, int sourceHeight,
+			 int targetLeft, int targetTop,
+			 char sourceChar, int attributes);
+
+	// Get the current state of CapsLock and NumLock.
+	[MethodImpl(MethodImplOptions.InternalCall)]
+	extern public static int GetLockState();
+
+	// Flags that may be returned by "GetLockState".
+	public const int CapsLock = 0x01;
+	public const int NumLock  = 0x02;
+
+	// Get the current size of the cursor.  The default value is usually 25.
+	[MethodImpl(MethodImplOptions.InternalCall)]
+	extern public static int GetCursorSize();
+
+	// Set the current size of the cursor.  May be ignored if it isn't
+	// possible to set the cursor size.
+	[MethodImpl(MethodImplOptions.InternalCall)]
+	extern public static void SetCursorSize(int size);
+
+	// Get a flag indicating if the cursor is visible.
+	[MethodImpl(MethodImplOptions.InternalCall)]
+	extern public static bool GetCursorVisible();
+
+	// Set a flag indicating if the cursor is visible.  May be ignored.
+	[MethodImpl(MethodImplOptions.InternalCall)]
+	extern public static void SetCursorVisible(bool flag);
+
 }; // class Stdio
 
 }; // namespace Platform
