@@ -53,20 +53,13 @@ public class ArrayList : ICloneable, ICollection, IEnumerable, IList
 	// Construct an array list from the contents of a collection.
 	public ArrayList(ICollection c)
 			{
-				IEnumerator enumerator;
-				int index;
 				if(c == null)
 				{
 					throw new ArgumentNullException("c");
 				}
-				count = c.Count;
-				store = new Object [count];
-				enumerator = c.GetEnumerator();
-				for(index = 0; enumerator.MoveNext(); ++index)
-				{
-					store[index] = enumerator.Current;
-				}
-				generation = 0;
+				count = 0;
+				store = new Object[c.Count];
+				this.InsertRange( count, c );
 			}
 
 	// Reallocate the array to accomodate "n" new entries at "index".
@@ -81,6 +74,7 @@ public class ArrayList : ICloneable, ICollection, IEnumerable, IList
 					while(posn >= index)
 					{
 						store[posn + n] = store[posn];
+						store[posn] = null; // Brubbel: set the new free entry to zero
 						--posn;
 					}
 				}
@@ -111,9 +105,15 @@ public class ArrayList : ICloneable, ICollection, IEnumerable, IList
 	// This modifies "count".
 	private void Delete(int n, int index)
 			{
-				while((index + n) < count)
+				while((index + n) <= count) // brubbel : must be <=
 				{
-					store[index] = store[index + n];
+					if( (index + n) == count ) {
+						store[index] = null;
+					}
+					else {
+						store[index] = store[index + n];
+						store[index + n] = null; // brubbel : set the new free entries to zero to avoid mem leaks
+					}
 					++index;
 				}
 				count -= n;
@@ -239,35 +239,13 @@ public class ArrayList : ICloneable, ICollection, IEnumerable, IList
 	// Add the contents of a collection as a range.
 	public virtual void AddRange(ICollection c)
 			{
-				int cCount;
-				IEnumerator enumerator;
-				if(c == null)
-				{
-					throw new ArgumentNullException("c");
-				}
-				cCount = c.Count;
-				if((count + cCount) > store.Length)
-				{
-					Realloc(cCount, count);
-				}
-				enumerator = c.GetEnumerator();
-				
-				if(enumerator.MoveNext())
-				{
-					++generation;
-					do
-					{
-						store[count++] = enumerator.Current;
-					}
-					while(enumerator.MoveNext());
-				}
+				// use nsert range to ensure the SyncRoot check
+				this.InsertRange( count, c );
 			}
 
 	// Insert the contents of a collection as a range.
 	public virtual void InsertRange(int index, ICollection c)
 			{
-				int cCount;
-				IEnumerator enumerator;
 				if(c == null)
 				{
 					throw new ArgumentNullException("c");
@@ -277,25 +255,26 @@ public class ArrayList : ICloneable, ICollection, IEnumerable, IList
 					throw new ArgumentOutOfRangeException
 						("index", _("ArgRange_Array"));
 				}
-				cCount = c.Count;
-				Realloc(cCount, index);
-				enumerator = c.GetEnumerator();
+				int cCount = c.Count;
 				
-				if(c.SyncRoot == this && (c is ICloneable))
-				{
-					c = (ICollection)((ICloneable)c).Clone();
-				}
-				
-				if(enumerator.MoveNext())
-				{
-					++generation;
-					do
+				if( cCount > 0 ) {
+					if((count + cCount) > store.Length)
 					{
-						store[index++] = enumerator.Current;
+						Realloc(cCount, count);
 					}
-					while(enumerator.MoveNext());
+					if( index < this.count ) {
+						Array.Copy( this.store, index, this.store, index+cCount, this.count - index ); 
+					}
+					if( c.SyncRoot == this.SyncRoot ) {
+						Array.Copy( this.store, 0, this.store, index, index );
+						Array.Copy( this.store, index+cCount, this.store, 2*index, this.count - index );
+					}
+					else {
+						c.CopyTo( this.store, index );
+					}
+					this.count += cCount;
+					++generation;
 				}
-				count += cCount;
 			}
 
 	// Remove a range of elements from an array list.
@@ -361,7 +340,7 @@ public class ArrayList : ICloneable, ICollection, IEnumerable, IList
 					{
 						if(comparer != null)
 						{
-							cmp = comparer.Compare(value, elem);
+							cmp = comparer.Compare(elem, value);
 						}
 						else if((icmp = (elem as IComparable)) != null)
 						{
@@ -418,6 +397,15 @@ public class ArrayList : ICloneable, ICollection, IEnumerable, IList
 	// Implement the ICollection interface.
 	public virtual void CopyTo(Array array, int arrayIndex)
 			{
+				// addidtonal checks to throw the correct exception
+				if(array == null)
+				{
+					throw new ArgumentNullException("array");
+				}
+				if(array.Rank > 1)
+				{
+					throw new ArgumentException("array", _("Arg_RankMustBe1"));
+				}
 				Array.Copy(store, 0, array, arrayIndex, count);
 			}
 	public virtual int Count
@@ -650,28 +638,20 @@ public class ArrayList : ICloneable, ICollection, IEnumerable, IList
 	// Set a range of array list elements to the members of a collection.
 	public virtual void SetRange(int index, ICollection c)
 			{
-				int count;
-				IEnumerator enumerator;
 				if(c == null)
 				{
 					throw new ArgumentNullException("c");
 				}
-				if(index < 0)
-				{
+				int cCount = c.Count;
+				if( (index < 0) || (index > (this.count - cCount ) ) ) {
 					throw new ArgumentOutOfRangeException
-						("index", _("ArgRange_Array"));
+							("index", _("ArgRange_Array"));
 				}
-				count = c.Count;
-				if((this.count - index) < count)
-				{
-					throw new ArgumentException(_("Arg_InvalidArrayRange"));
+				
+				if( cCount > 0 ) {
+					c.CopyTo( this.store, index );
+					++generation;
 				}
-				enumerator = c.GetEnumerator();
-				while(enumerator.MoveNext())
-				{
-					store[index++] = enumerator.Current;
-				}
-				++generation;
 			}
 
 	// Inner version of "Sort".
