@@ -178,6 +178,10 @@ public class SortedList : IDictionary, ICollection, IEnumerable, ICloneable
 					keys[posn]   = keys[posn + 1];
 					values[posn] = values[posn + 1];
 				}
+				// remove last reference to avoid memory leak
+				keys[posn]   = null;
+				values[posn] = null;
+				
 				--count;
 				++generation;
 			}
@@ -198,6 +202,8 @@ public class SortedList : IDictionary, ICollection, IEnumerable, ICloneable
 			}
 	public virtual void Clear()
 			{
+				Array.Clear( values, 0, values.Length ); // clear the array to release references
+				Array.Clear( keys, 0, keys.Length ); // clear the array to release references
 				count = 0;
 				++generation;
 			}
@@ -1147,6 +1153,7 @@ public class SortedList : IDictionary, ICollection, IEnumerable, ICloneable
 				{
 					this.list  = list;
 					generation = list.generation;
+					Reset();
 				}
 
 		// Implement the IEnumerator interface.
@@ -1207,6 +1214,7 @@ public class SortedList : IDictionary, ICollection, IEnumerable, ICloneable
 				{
 					this.list  = list;
 					generation = list.generation;
+					Reset();
 				}
 
 		// Implement the IEnumerator interface.
