@@ -1,7 +1,7 @@
 /*
  * CultureInfo.cs - Implementation of "System.Globalization.CultureInfo".
  *
- * Copyright (C) 2001, 2003  Southern Storm Software, Pty Ltd.
+ * Copyright (C) 2001, 2003, 2004  Southern Storm Software, Pty Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -111,8 +111,13 @@ public class CultureInfo : ICloneable, IFormatProvider
 				}
 			#if CONFIG_REFLECTION
 				userOverride = useUserOverride;
-				handler = GetCultureHandler(name, useUserOverride);
-				cultureID = handler.LCID;
+				if( name == string.Empty ) {
+					cultureID = 0x007F;
+				}
+				else {
+					handler = GetCultureHandler(name, useUserOverride);
+					cultureID = handler.LCID;
+				}
 			#else
 				cultureID = 0x007F;
 				userOverride = useUserOverride;
@@ -241,11 +246,10 @@ public class CultureInfo : ICloneable, IFormatProvider
 	// Set the current culture.
 	internal static void SetCurrentCulture(CultureInfo value)
 			{
-				if(value == null)
-				{
-					throw new ArgumentNullException("value");
-				}
 				currentCulture = value;
+			#if !ECMA_COMPAT
+				RegionInfo.currentRegion = null;	// Recreate on next call.
+			#endif
 			}
 
 	// Get the current UI culture object for the running thread.
@@ -264,10 +268,6 @@ public class CultureInfo : ICloneable, IFormatProvider
 	// Set the current UI culture.
 	internal static void SetCurrentUICulture(CultureInfo value)
 			{
-				if(value == null)
-				{
-					throw new ArgumentNullException("value");
-				}
 				currentUICulture = value;
 			}
 
@@ -534,6 +534,23 @@ public class CultureInfo : ICloneable, IFormatProvider
 				}
 			}
 
+#if CONFIG_FRAMEWORK_1_2
+
+	// Get the keyboard layout identifier for this culture.
+	public virtual int KeyboardLayoutID
+			{
+				get
+				{
+					if(handler != null)
+					{
+						return handler.KeyboardLayoutID;
+					}
+					return 0x0C09;	// US 101 keyboard.
+				}
+			}
+
+#endif // CONFIG_FRAMEWORK_1_2
+
 	// Get or set the number formatting information for this culture.
 	public virtual NumberFormatInfo NumberFormat
 			{
@@ -711,7 +728,22 @@ public class CultureInfo : ICloneable, IFormatProvider
 	// Implementation of the ICloneable interface.
 	public virtual Object Clone()
 			{
+			#if !ECMA_COMPAT
+				CultureInfo culture = (CultureInfo)(MemberwiseClone());
+				culture.readOnly = false;
+				// clone DateTimeFormat and NumberFormat if available
+				if(dateTimeFormat != null)
+				{
+					culture.dateTimeFormat = (DateTimeFormatInfo)dateTimeFormat.Clone();
+				}
+				if(numberFormat != null)
+				{
+					culture.numberFormat = (NumberFormatInfo)numberFormat.Clone();
+				}
+				return culture;
+			#else
 				return MemberwiseClone();
+			#endif
 			}
 
 	// Implementation of the IFormatProvider interface.
