@@ -156,11 +156,11 @@ public abstract class MulticastDelegate : Delegate
 				{
 					list = list.prev;
 					d = d.prev;
-					if(((Object)list) == null)
+					if(((Object)d) == null)
 					{
-						return (((Object)d) == null);
+						return true;
 					}
-					else if(((Object)d) == null)
+					if(((Object)list) == null)
 					{
 						return false;
 					}
