@@ -2,7 +2,7 @@
  * CompilationRelaxationsAttribute.cs - Implementation of the
  *	"System.Runtime.CompilerServices.CompilationRelaxationsAttribute" class.
  *
- * Copyright (C) 2001  Southern Storm Software, Pty Ltd.
+ * Copyright (C) 2001, 2004  Southern Storm Software, Pty Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,12 @@ public class CompilationRelaxationsAttribute : Attribute
 			{
 				value = relaxations;
 			}
+#if CONFIG_FRAMEWORK_1_2
+	public CompilationRelaxationsAttribute(CompilationRelaxations relaxations)
+			{
+				value = (int)relaxations;
+			}
+#endif // CONFIG_FRAMEWORK_1_2
 
 	// Properties.
 	public int CompilationRelaxations
